package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.util.ObjectUtil;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;

import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.tourism.webadmin.back.dto.ChinaAreaDto;
import com.tourism.webadmin.back.model.ChinaArea;
import com.tourism.webadmin.back.service.ChinaAreaService;
import com.tourism.webadmin.back.vo.ChinaAreaVo;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 中国行政区划信息操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "中国行政区划信息管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/chinaArea")
public class ChinaAreaController {

    @Autowired
    private ChinaAreaService chinaAreaService;

    /**
     * 新增中国行政区划信息数据。
     *
     * @param chinaAreaDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"chinaAreaDto.code", "chinaAreaDto.searchString"})
    @SaCheckPermission("chinaArea.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody ChinaAreaDto chinaAreaDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(chinaAreaDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        ChinaArea chinaArea = MyModelUtil.copyTo(chinaAreaDto, ChinaArea.class);
        // 验证父Id的数据合法性
        ChinaArea parentChinaArea;
        if (MyCommonUtil.isNotBlankOrNull(chinaArea.getCityCode())) {
            parentChinaArea = chinaAreaService.getById(chinaArea.getCityCode());
            if (parentChinaArea == null) {
                errorMessage = "数据验证失败，关联的父节点并不存在，请刷新后重试！";
                return ResponseResult.error(ErrorCodeEnum.DATA_PARENT_ID_NOT_EXIST, errorMessage);
            }
        }
        chinaArea = chinaAreaService.saveNew(chinaArea);
        return ResponseResult.success(chinaArea.getCode());
    }

    /**
     * 更新中国行政区划信息数据。
     *
     * @param chinaAreaDto 更新对象。
     * @return 应答结果对象。
     */
    @ApiOperationSupport(ignoreParameters = {"chinaAreaDto.searchString"})
    @SaCheckPermission("chinaArea.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody ChinaAreaDto chinaAreaDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(chinaAreaDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        ChinaArea chinaArea = MyModelUtil.copyTo(chinaAreaDto, ChinaArea.class);
        ChinaArea originalChinaArea = chinaAreaService.getById(chinaArea.getCode());
        if (originalChinaArea == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        // 验证父Id的数据合法性
        if (MyCommonUtil.isNotBlankOrNull(chinaArea.getCityCode())
                && ObjectUtil.notEqual(chinaArea.getCityCode(), originalChinaArea.getCityCode())) {
            ChinaArea parentChinaArea = chinaAreaService.getById(chinaArea.getCityCode());
            if (parentChinaArea == null) {
                // NOTE: 修改下面方括号中的话述
                errorMessage = "数据验证失败，关联的 [父节点] 并不存在，请刷新后重试！";
                return ResponseResult.error(ErrorCodeEnum.DATA_PARENT_ID_NOT_EXIST, errorMessage);
            }
        }
        if (!chinaAreaService.update(chinaArea, originalChinaArea)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除中国行政区划信息数据。
     *
     * @param code 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("chinaArea.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long code) {
        if (MyCommonUtil.existBlankArgument(code)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(code);
    }

    /**
     * 批量删除中国行政区划信息数据。
     *
     * @param codeList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("chinaArea.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> codeList) {
        if (MyCommonUtil.existBlankArgument(codeList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long code : codeList) {
            ResponseResult<Void> responseResult = this.doDelete(code);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的中国行政区划信息列表。
     *
     * @param chinaAreaDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("chinaArea.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<ChinaAreaVo>> list(
            @MyRequestBody ChinaAreaDto chinaAreaDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        ChinaArea chinaAreaFilter = MyModelUtil.copyTo(chinaAreaDtoFilter, ChinaArea.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, ChinaArea.class);
        List<ChinaArea> chinaAreaList = chinaAreaService.getChinaAreaListWithRelation(chinaAreaFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(chinaAreaList, ChinaAreaVo.class));
    }

    /**
     * 查看指定中国行政区划信息对象详情。
     *
     * @param code 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("chinaArea.view")
    @GetMapping("/view")
    public ResponseResult<ChinaAreaVo> view(@RequestParam Long code) {
        ChinaArea chinaArea = chinaAreaService.getByIdWithRelation(code, MyRelationParam.full());
        if (chinaArea == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        ChinaAreaVo chinaAreaVo = MyModelUtil.copyTo(chinaArea, ChinaAreaVo.class);
        return ResponseResult.success(chinaAreaVo);
    }

    /**
     * 以字典形式返回全部中国行政区划信息数据集合。字典的键值为[code, name]。
     * 白名单接口，登录用户均可访问。
     *
     * @param filter 过滤对象。
     * @return 应答结果对象，包含的数据为 List<Map<String, String>>，map中包含两条记录，key的值分别是id和name，value对应具体数据。
     */
    @GetMapping("/listDict")
    public ResponseResult<List<Map<String, Object>>> listDict(@ParameterObject ChinaAreaDto filter) {
        List<ChinaArea> resultList =
                chinaAreaService.getListByFilter(MyModelUtil.copyTo(filter, ChinaArea.class));
        return ResponseResult.success(MyCommonUtil.toDictDataList(
                resultList, ChinaArea::getCode, ChinaArea::getName, ChinaArea::getCityCode));
    }

    /**
     * 根据字典Id集合，获取查询后的字典数据。
     *
     * @param dictIds 字典Id集合。
     * @return 应答结果对象，包含字典形式的数据集合。
     */
    @GetMapping("/listDictByIds")
    public ResponseResult<List<Map<String, Object>>> listDictByIds(@RequestParam List<Long> dictIds) {
        List<ChinaArea> resultList = chinaAreaService.getInList(new HashSet<>(dictIds));
        return ResponseResult.success(MyCommonUtil.toDictDataList(
                resultList, ChinaArea::getCode, ChinaArea::getName, ChinaArea::getCityCode));
    }

    /**
     * 根据父主键Id，以字典的形式返回其下级数据列表。
     * 白名单接口，登录用户均可访问。
     *
     * @param parentId 父主键Id。
     * @return 按照字典的形式返回下级数据列表。
     */
    @GetMapping("/listDictByParentId")
    public ResponseResult<List<Map<String, Object>>> listDictByParentId(@RequestParam(required = false) Long parentId) {
        List<ChinaArea> resultList = chinaAreaService.getListByParentId("cityCode", parentId);
        return ResponseResult.success(MyCommonUtil.toDictDataList(
                resultList, ChinaArea::getCode, ChinaArea::getName, ChinaArea::getCityCode));
    }

    private ResponseResult<Void> doDelete(Long code) {
        String errorMessage;
        // 验证关联Id的数据合法性
        ChinaArea originalChinaArea = chinaAreaService.getById(code);
        if (originalChinaArea == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (chinaAreaService.hasChildren(code)) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象存在子对象] ，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.HAS_CHILDREN_DATA, errorMessage);
        }
        if (!chinaAreaService.remove(code)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
