package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;

import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.tourism.webadmin.back.dto.CountryCodeDto;
import com.tourism.webadmin.back.model.CountryCode;
import com.tourism.webadmin.back.service.CountryCodeService;
import com.tourism.webadmin.back.vo.CountryCodeVo;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 国家地区编码表操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "国家地区编码表管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/countryCode")
public class CountryCodeController {

    @Autowired
    private CountryCodeService countryCodeService;

    /**
     * 新增国家地区编码表数据。
     *
     * @param countryCodeDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"countryCodeDto.id"})
    @SaCheckPermission("countryCode.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody CountryCodeDto countryCodeDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(countryCodeDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        CountryCode countryCode = MyModelUtil.copyTo(countryCodeDto, CountryCode.class);
        countryCode = countryCodeService.saveNew(countryCode);
        return ResponseResult.success(countryCode.getId());
    }

    /**
     * 更新国家地区编码表数据。
     *
     * @param countryCodeDto 更新对象。
     * @return 应答结果对象。
     */
    @SaCheckPermission("countryCode.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody CountryCodeDto countryCodeDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(countryCodeDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        CountryCode countryCode = MyModelUtil.copyTo(countryCodeDto, CountryCode.class);
        CountryCode originalCountryCode = countryCodeService.getById(countryCode.getId());
        if (originalCountryCode == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!countryCodeService.update(countryCode, originalCountryCode)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除国家地区编码表数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("countryCode.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除国家地区编码表数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("countryCode.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的国家地区编码表列表。
     *
     * @param countryCodeDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("countryCode.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<CountryCodeVo>> list(
            @MyRequestBody CountryCodeDto countryCodeDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        CountryCode countryCodeFilter = MyModelUtil.copyTo(countryCodeDtoFilter, CountryCode.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, CountryCode.class);
        List<CountryCode> countryCodeList =
                countryCodeService.getCountryCodeListWithRelation(countryCodeFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(countryCodeList, CountryCodeVo.class));
    }

    /**
     * 查看指定国家地区编码表对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("countryCode.view")
    @GetMapping("/view")
    public ResponseResult<CountryCodeVo> view(@RequestParam Long id) {
        CountryCode countryCode = countryCodeService.getByIdWithRelation(id, MyRelationParam.full());
        if (countryCode == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        CountryCodeVo countryCodeVo = MyModelUtil.copyTo(countryCode, CountryCodeVo.class);
        return ResponseResult.success(countryCodeVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        CountryCode originalCountryCode = countryCodeService.getById(id);
        if (originalCountryCode == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!countryCodeService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
