package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.util.ObjectUtil;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 包车区域菜单操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "包车区域菜单管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourCarContractMenu")
public class TourCarContractMenuController {

    @Autowired
    private TourCarContractMenuService tourCarContractMenuService;

    /**
     * 新增包车区域菜单数据。
     *
     * @param tourCarContractMenuDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"tourCarContractMenuDto.id", "tourCarContractMenuDto.searchString"})
    @SaCheckPermission("tourCarContractMenu.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody TourCarContractMenuDto tourCarContractMenuDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourCarContractMenuDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourCarContractMenu tourCarContractMenu = MyModelUtil.copyTo(tourCarContractMenuDto, TourCarContractMenu.class);
        // 验证父Id的数据合法性
        TourCarContractMenu parentTourCarContractMenu;
        if (MyCommonUtil.isNotBlankOrNull(tourCarContractMenu.getParentId())) {
            parentTourCarContractMenu = tourCarContractMenuService.getById(tourCarContractMenu.getParentId());
            if (parentTourCarContractMenu == null) {
                errorMessage = "数据验证失败，关联的父节点并不存在，请刷新后重试！";
                return ResponseResult.error(ErrorCodeEnum.DATA_PARENT_ID_NOT_EXIST, errorMessage);
            }
        }
        tourCarContractMenu = tourCarContractMenuService.saveNew(tourCarContractMenu);
        return ResponseResult.success(tourCarContractMenu.getId());
    }

    /**
     * 更新包车区域菜单数据。
     *
     * @param tourCarContractMenuDto 更新对象。
     * @return 应答结果对象。
     */
    @ApiOperationSupport(ignoreParameters = {"tourCarContractMenuDto.searchString"})
    @SaCheckPermission("tourCarContractMenu.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourCarContractMenuDto tourCarContractMenuDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourCarContractMenuDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourCarContractMenu tourCarContractMenu = MyModelUtil.copyTo(tourCarContractMenuDto, TourCarContractMenu.class);
        TourCarContractMenu originalTourCarContractMenu = tourCarContractMenuService.getById(tourCarContractMenu.getId());
        if (originalTourCarContractMenu == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        // 验证父Id的数据合法性
        if (MyCommonUtil.isNotBlankOrNull(tourCarContractMenu.getParentId())
                && ObjectUtil.notEqual(tourCarContractMenu.getParentId(), originalTourCarContractMenu.getParentId())) {
            TourCarContractMenu parentTourCarContractMenu = tourCarContractMenuService.getById(tourCarContractMenu.getParentId());
            if (parentTourCarContractMenu == null) {
                // NOTE: 修改下面方括号中的话述
                errorMessage = "数据验证失败，关联的 [父节点] 并不存在，请刷新后重试！";
                return ResponseResult.error(ErrorCodeEnum.DATA_PARENT_ID_NOT_EXIST, errorMessage);
            }
        }
        if (!tourCarContractMenuService.update(tourCarContractMenu, originalTourCarContractMenu)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除包车区域菜单数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourCarContractMenu.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除包车区域菜单数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourCarContractMenu.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的包车区域菜单列表。
     *
     * @param tourCarContractMenuDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourCarContractMenu.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourCarContractMenuVo>> list(
            @MyRequestBody TourCarContractMenuDto tourCarContractMenuDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourCarContractMenu tourCarContractMenuFilter = MyModelUtil.copyTo(tourCarContractMenuDtoFilter, TourCarContractMenu.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourCarContractMenu.class);
        List<TourCarContractMenu> tourCarContractMenuList =
                tourCarContractMenuService.getTourCarContractMenuListWithRelation(tourCarContractMenuFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourCarContractMenuList, TourCarContractMenuVo.class));
    }

    /**
     * 查看指定包车区域菜单对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourCarContractMenu.view")
    @GetMapping("/view")
    public ResponseResult<TourCarContractMenuVo> view(@RequestParam Long id) {
        TourCarContractMenu tourCarContractMenu = tourCarContractMenuService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourCarContractMenu == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourCarContractMenuVo tourCarContractMenuVo = MyModelUtil.copyTo(tourCarContractMenu, TourCarContractMenuVo.class);
        return ResponseResult.success(tourCarContractMenuVo);
    }

    /**
     * 以字典形式返回全部包车区域菜单数据集合。字典的键值为[id, menuName]。
     * 白名单接口，登录用户均可访问。
     *
     * @param filter 过滤对象。
     * @return 应答结果对象，包含的数据为 List<Map<String, String>>，map中包含两条记录，key的值分别是id和name，value对应具体数据。
     */
    @GetMapping("/listDict")
    public ResponseResult<List<Map<String, Object>>> listDict(@ParameterObject TourCarContractMenuDto filter) {
        List<TourCarContractMenu> resultList =
                tourCarContractMenuService.getListByFilter(MyModelUtil.copyTo(filter, TourCarContractMenu.class));
        return ResponseResult.success(MyCommonUtil.toDictDataList(
                resultList, TourCarContractMenu::getId, TourCarContractMenu::getMenuName, TourCarContractMenu::getParentId));
    }

    /**
     * 根据字典Id集合，获取查询后的字典数据。
     *
     * @param dictIds 字典Id集合。
     * @return 应答结果对象，包含字典形式的数据集合。
     */
    @GetMapping("/listDictByIds")
    public ResponseResult<List<Map<String, Object>>> listDictByIds(@RequestParam List<Long> dictIds) {
        List<TourCarContractMenu> resultList = tourCarContractMenuService.getInList(new HashSet<>(dictIds));
        return ResponseResult.success(MyCommonUtil.toDictDataList(
                resultList, TourCarContractMenu::getId, TourCarContractMenu::getMenuName, TourCarContractMenu::getParentId));
    }

    /**
     * 根据父主键Id，以字典的形式返回其下级数据列表。
     * 白名单接口，登录用户均可访问。
     *
     * @param parentId 父主键Id。
     * @return 按照字典的形式返回下级数据列表。
     */
    @GetMapping("/listDictByParentId")
    public ResponseResult<List<Map<String, Object>>> listDictByParentId(@RequestParam(required = false) Long parentId) {
        List<TourCarContractMenu> resultList = tourCarContractMenuService.getListByParentId("parentId", parentId);
        return ResponseResult.success(MyCommonUtil.toDictDataList(
                resultList, TourCarContractMenu::getId, TourCarContractMenu::getMenuName, TourCarContractMenu::getParentId));
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourCarContractMenu originalTourCarContractMenu = tourCarContractMenuService.getById(id);
        if (originalTourCarContractMenu == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (tourCarContractMenuService.hasChildren(id)) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象存在子对象] ，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.HAS_CHILDREN_DATA, errorMessage);
        }
        if (!tourCarContractMenuService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
