package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.util.QueryConditionProccessUtils;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 包车订单操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "包车订单管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourCarOrder")
public class TourCarOrderController {

    @Autowired
    private TourCarOrderService tourCarOrderService;

    /**
     * 新增包车订单数据。
     *
     * @param tourCarOrderDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {
            "tourCarOrderDto.id",
            "tourCarOrderDto.searchString",
            "tourCarOrderDto.startTimeStart",
            "tourCarOrderDto.startTimeEnd",
            "tourCarOrderDto.endTimeStart",
            "tourCarOrderDto.endTimeEnd"})
    @SaCheckPermission("tourCarOrder.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody TourCarOrderDto tourCarOrderDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourCarOrderDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourCarOrder tourCarOrder = MyModelUtil.copyTo(tourCarOrderDto, TourCarOrder.class);
        // 验证关联Id的数据合法性
        CallResult callResult = tourCarOrderService.verifyRelatedData(tourCarOrder, null);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        tourCarOrder = tourCarOrderService.saveNew(tourCarOrder);
        return ResponseResult.success(tourCarOrder.getId());
    }

    /**
     * 更新包车订单数据。
     *
     * @param tourCarOrderDto 更新对象。
     * @return 应答结果对象。
     */
    @ApiOperationSupport(ignoreParameters = {
            "tourCarOrderDto.searchString",
            "tourCarOrderDto.startTimeStart",
            "tourCarOrderDto.startTimeEnd",
            "tourCarOrderDto.endTimeStart",
            "tourCarOrderDto.endTimeEnd"})
    @SaCheckPermission("tourCarOrder.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourCarOrderDto tourCarOrderDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourCarOrderDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }

        tourCarOrderDto.setEndTime(QueryConditionProccessUtils.getEndDateTime(tourCarOrderDto.getStartTime(), tourCarOrderDto.getDays()));
        TourCarOrder tourCarOrder = MyModelUtil.copyTo(tourCarOrderDto, TourCarOrder.class);
        TourCarOrder originalTourCarOrder = tourCarOrderService.getById(tourCarOrder.getId());
        if (originalTourCarOrder == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        // 验证关联Id的数据合法性
        CallResult callResult = tourCarOrderService.verifyRelatedData(tourCarOrder, originalTourCarOrder);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        if (!tourCarOrderService.update(tourCarOrder, originalTourCarOrder)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除包车订单数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourCarOrder.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除包车订单数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourCarOrder.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的包车订单列表。
     *
     * @param tourCarOrderDtoFilter 过滤对象。
     * @param tourCarCategoryDtoFilter 一对一从表过滤对象。
     * @param tourCarDriverDtoFilter 一对一从表过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourCarOrder.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourCarOrderVo>> list(
            @MyRequestBody TourCarOrderDto tourCarOrderDtoFilter,
            @MyRequestBody TourCarCategoryDto tourCarCategoryDtoFilter,
            @MyRequestBody TourCarDriverDto tourCarDriverDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourCarOrder tourCarOrderFilter = MyModelUtil.copyTo(tourCarOrderDtoFilter, TourCarOrder.class);
        TourCarCategory tourCarCategoryFilter = MyModelUtil.copyTo(tourCarCategoryDtoFilter, TourCarCategory.class);
        TourCarDriver tourCarDriverFilter = MyModelUtil.copyTo(tourCarDriverDtoFilter, TourCarDriver.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourCarOrder.class);
        List<TourCarOrder> tourCarOrderList =
                tourCarOrderService.getTourCarOrderListWithRelation(tourCarOrderFilter, tourCarCategoryFilter, tourCarDriverFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourCarOrderList, TourCarOrderVo.class));
    }

    /**
     * 查看指定包车订单对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourCarOrder.view")
    @GetMapping("/view")
    public ResponseResult<TourCarOrderVo> view(@RequestParam Long id) {
        TourCarOrder tourCarOrder = tourCarOrderService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourCarOrder == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourCarOrderVo tourCarOrderVo = MyModelUtil.copyTo(tourCarOrder, TourCarOrderVo.class);
        return ResponseResult.success(tourCarOrderVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourCarOrder originalTourCarOrder = tourCarOrderService.getById(id);
        if (originalTourCarOrder == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourCarOrderService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
