package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.tourism.common.additional.config.ApplicationConfig;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;

/**
 * 房屋知识信息管理操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "房屋知识信息管理管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourHoeseKnowledgeInfo")
public class TourHoeseKnowledgeInfoController {

    @Autowired
    private ApplicationConfig appConfig;
    @Autowired
    private TourHoeseKnowledgeInfoService tourHoeseKnowledgeInfoService;

    /**
     * 新增房屋知识信息管理数据。
     *
     * @param tourHoeseKnowledgeInfoDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"tourHoeseKnowledgeInfoDto.id", "tourHoeseKnowledgeInfoDto.searchString"})
    @SaCheckPermission("tourHoeseKnowledgeInfo.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody TourHoeseKnowledgeInfoDto tourHoeseKnowledgeInfoDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourHoeseKnowledgeInfoDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourHoeseKnowledgeInfo tourHoeseKnowledgeInfo = MyModelUtil.copyTo(tourHoeseKnowledgeInfoDto, TourHoeseKnowledgeInfo.class);
        // 验证关联Id的数据合法性
        CallResult callResult = tourHoeseKnowledgeInfoService.verifyRelatedData(tourHoeseKnowledgeInfo, null);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        tourHoeseKnowledgeInfo = tourHoeseKnowledgeInfoService.saveNew(tourHoeseKnowledgeInfo);
        return ResponseResult.success(tourHoeseKnowledgeInfo.getId());
    }

    /**
     * 更新房屋知识信息管理数据。
     *
     * @param tourHoeseKnowledgeInfoDto 更新对象。
     * @return 应答结果对象。
     */
    @ApiOperationSupport(ignoreParameters = {"tourHoeseKnowledgeInfoDto.searchString"})
    @SaCheckPermission("tourHoeseKnowledgeInfo.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourHoeseKnowledgeInfoDto tourHoeseKnowledgeInfoDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourHoeseKnowledgeInfoDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourHoeseKnowledgeInfo tourHoeseKnowledgeInfo = MyModelUtil.copyTo(tourHoeseKnowledgeInfoDto, TourHoeseKnowledgeInfo.class);
        TourHoeseKnowledgeInfo originalTourHoeseKnowledgeInfo = tourHoeseKnowledgeInfoService.getById(tourHoeseKnowledgeInfo.getId());
        if (originalTourHoeseKnowledgeInfo == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        // 验证关联Id的数据合法性
        CallResult callResult = tourHoeseKnowledgeInfoService.verifyRelatedData(tourHoeseKnowledgeInfo, originalTourHoeseKnowledgeInfo);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        if (!tourHoeseKnowledgeInfoService.update(tourHoeseKnowledgeInfo, originalTourHoeseKnowledgeInfo)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除房屋知识信息管理数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourHoeseKnowledgeInfo.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除房屋知识信息管理数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourHoeseKnowledgeInfo.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的房屋知识信息管理列表。
     *
     * @param tourHoeseKnowledgeInfoDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourHoeseKnowledgeInfo.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourHoeseKnowledgeInfoVo>> list(
            @MyRequestBody TourHoeseKnowledgeInfoDto tourHoeseKnowledgeInfoDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourHoeseKnowledgeInfo tourHoeseKnowledgeInfoFilter = MyModelUtil.copyTo(tourHoeseKnowledgeInfoDtoFilter, TourHoeseKnowledgeInfo.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourHoeseKnowledgeInfo.class);
        List<TourHoeseKnowledgeInfo> tourHoeseKnowledgeInfoList =
                tourHoeseKnowledgeInfoService.getTourHoeseKnowledgeInfoListWithRelation(tourHoeseKnowledgeInfoFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourHoeseKnowledgeInfoList, TourHoeseKnowledgeInfoVo.class));
    }

    /**
     * 导入主表数据列表。
     *
     * @param importFile 上传的文件，目前仅仅支持xlsx和xls两种格式。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourHoeseKnowledgeInfo.import")
    @OperationLog(type = SysOperationLogType.IMPORT)
    @PostMapping("/import")
    public ResponseResult<Void> importBatch(
            @RequestParam Boolean skipHeader,
            @RequestParam("importFile") MultipartFile importFile) throws IOException {
        String filename = ImportUtil.saveImportFile(appConfig.getUploadFileBaseDir(), null, importFile);
        // 这里可以指定需要忽略导入的字段集合。如创建时间、创建人、更新时间、更新人、主键Id和逻辑删除，
        // 以及一些存在缺省值且无需导入的字段。其中主键字段和逻辑删除字段不需要在这里设置，批量插入逻辑会自动处理的。
        Set<String> ignoreFieldSet = new HashSet<>();
        ignoreFieldSet.add("createUserId");
        ignoreFieldSet.add("createTime");
        ignoreFieldSet.add("updateUserId");
        ignoreFieldSet.add("updateTime");
        List<ImportUtil.ImportHeaderInfo> headerInfoList = ImportUtil.makeHeaderInfoList(TourHoeseKnowledgeInfo.class, ignoreFieldSet);
        // 下面是导入时需要注意的地方，如果我们缺省生成的代码，与实际情况存在差异，请手动修改。
        // 1. 头信息数据字段，我们只是根据当前的主表实体对象生成了缺省数组，开发者可根据实际情况，对headerInfoList进行修改。
        ImportUtil.ImportHeaderInfo[] headerInfos = headerInfoList.toArray(new ImportUtil.ImportHeaderInfo[]{});
        // 2. 这里需要根据实际情况决定，导入文件中第一行是否为中文头信息，如果是可以跳过。这里我们默认为true。
        // 这里根据自己的实际需求，为doImport的最后一个参数，传递需要进行字典转换的字段集合。
        // 注意，集合中包含需要翻译的Java字段名，如: gradeId。
        Set<String> translatedDictFieldSet = new HashSet<>();
        List<TourHoeseKnowledgeInfo> dataList =
                ImportUtil.doImport(headerInfos, skipHeader, filename, TourHoeseKnowledgeInfo.class, translatedDictFieldSet);
        CallResult result = tourHoeseKnowledgeInfoService.verifyImportList(dataList, translatedDictFieldSet);
        if (!result.isSuccess()) {
            // result中返回了具体的验证失败对象，如果需要返回更加详细的错误，可根据实际情况手动修改。
            return ResponseResult.errorFrom(result);
        }
        tourHoeseKnowledgeInfoService.saveNewBatch(dataList, -1);
        return ResponseResult.success();
    }

    /**
     * 导出符合过滤条件的房屋知识信息管理列表。
     *
     * @param tourHoeseKnowledgeInfoDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @throws IOException 文件读写失败。
     */
    @SaCheckPermission("tourHoeseKnowledgeInfo.export")
    @OperationLog(type = SysOperationLogType.EXPORT, saveResponse = false)
    @PostMapping("/export")
    public void export(
            @MyRequestBody TourHoeseKnowledgeInfoDto tourHoeseKnowledgeInfoDtoFilter,
            @MyRequestBody MyOrderParam orderParam) throws IOException {
        TourHoeseKnowledgeInfo tourHoeseKnowledgeInfoFilter = MyModelUtil.copyTo(tourHoeseKnowledgeInfoDtoFilter, TourHoeseKnowledgeInfo.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourHoeseKnowledgeInfo.class);
        List<TourHoeseKnowledgeInfo> resultList =
                tourHoeseKnowledgeInfoService.getTourHoeseKnowledgeInfoListWithRelation(tourHoeseKnowledgeInfoFilter, orderBy);
        // 导出文件的标题数组
        // NOTE: 下面的代码中仅仅导出了主表数据，主表聚合计算数据和主表关联字典的数据。
        // 一对一从表数据的导出，可根据需要自行添加。如：headerMap.put("slaveFieldName.xxxField", "标题名称")
        Map<String, String> headerMap = new LinkedHashMap<>(10);
        headerMap.put("id", "主键");
        headerMap.put("typeDictMap.name", "知识类型 0买房 1装修");
        headerMap.put("title", "知识标题");
        headerMap.put("content", "知识内容");
        headerMap.put("enableDictMap.name", "是否启用，0禁用，1启用");
        headerMap.put("dataState", "删除标记(1: 正常 -1: 已删除)");
        headerMap.put("createUserId", "创建人");
        headerMap.put("createTime", "创建时间");
        headerMap.put("updateUserId", "修改人");
        headerMap.put("updateTime", "修改时间");
        ExportUtil.doExport(resultList, headerMap, "tourHoeseKnowledgeInfo.xlsx");
    }

    /**
     * 查看指定房屋知识信息管理对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourHoeseKnowledgeInfo.view")
    @GetMapping("/view")
    public ResponseResult<TourHoeseKnowledgeInfoVo> view(@RequestParam Long id) {
        TourHoeseKnowledgeInfo tourHoeseKnowledgeInfo = tourHoeseKnowledgeInfoService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourHoeseKnowledgeInfo == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourHoeseKnowledgeInfoVo tourHoeseKnowledgeInfoVo = MyModelUtil.copyTo(tourHoeseKnowledgeInfo, TourHoeseKnowledgeInfoVo.class);
        return ResponseResult.success(tourHoeseKnowledgeInfoVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourHoeseKnowledgeInfo originalTourHoeseKnowledgeInfo = tourHoeseKnowledgeInfoService.getById(id);
        if (originalTourHoeseKnowledgeInfo == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourHoeseKnowledgeInfoService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
