package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.util.ReflectUtil;
import com.tourism.common.core.upload.BaseUpDownloader;
import com.tourism.common.core.upload.UpDownloaderFactory;
import com.tourism.common.core.upload.UploadResponseInfo;
import com.tourism.common.core.upload.UploadStoreInfo;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.tourism.common.redis.cache.SessionCacheHelper;
import com.tourism.common.additional.config.ApplicationConfig;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import jakarta.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.*;

/**
 * 房屋顾问信息管理操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "房屋顾问信息管理管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourHouseAdvisorInfo")
public class TourHouseAdvisorInfoController {

    @Autowired
    private ApplicationConfig appConfig;
    @Autowired
    private SessionCacheHelper cacheHelper;
    @Autowired
    private UpDownloaderFactory upDownloaderFactory;
    @Autowired
    private TourHouseAdvisorInfoService tourHouseAdvisorInfoService;

    /**
     * 新增房屋顾问信息管理数据。
     *
     * @param tourHouseAdvisorInfoDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {
            "tourHouseAdvisorInfoDto.id",
            "tourHouseAdvisorInfoDto.searchString",
            "tourHouseAdvisorInfoDto.experienceAgeStart",
            "tourHouseAdvisorInfoDto.experienceAgeEnd",
            "tourHouseAdvisorInfoDto.successfulDeliveryCountStart",
            "tourHouseAdvisorInfoDto.successfulDeliveryCountEnd"})
    @SaCheckPermission("tourHouseAdvisorInfo.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody TourHouseAdvisorInfoDto tourHouseAdvisorInfoDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourHouseAdvisorInfoDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourHouseAdvisorInfo tourHouseAdvisorInfo = MyModelUtil.copyTo(tourHouseAdvisorInfoDto, TourHouseAdvisorInfo.class);
        // 验证关联Id的数据合法性
        CallResult callResult = tourHouseAdvisorInfoService.verifyRelatedData(tourHouseAdvisorInfo, null);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        tourHouseAdvisorInfo = tourHouseAdvisorInfoService.saveNew(tourHouseAdvisorInfo);
        return ResponseResult.success(tourHouseAdvisorInfo.getId());
    }

    /**
     * 更新房屋顾问信息管理数据。
     *
     * @param tourHouseAdvisorInfoDto 更新对象。
     * @return 应答结果对象。
     */
    @ApiOperationSupport(ignoreParameters = {
            "tourHouseAdvisorInfoDto.searchString",
            "tourHouseAdvisorInfoDto.experienceAgeStart",
            "tourHouseAdvisorInfoDto.experienceAgeEnd",
            "tourHouseAdvisorInfoDto.successfulDeliveryCountStart",
            "tourHouseAdvisorInfoDto.successfulDeliveryCountEnd"})
    @SaCheckPermission("tourHouseAdvisorInfo.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourHouseAdvisorInfoDto tourHouseAdvisorInfoDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourHouseAdvisorInfoDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourHouseAdvisorInfo tourHouseAdvisorInfo = MyModelUtil.copyTo(tourHouseAdvisorInfoDto, TourHouseAdvisorInfo.class);
        TourHouseAdvisorInfo originalTourHouseAdvisorInfo = tourHouseAdvisorInfoService.getById(tourHouseAdvisorInfo.getId());
        if (originalTourHouseAdvisorInfo == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        // 验证关联Id的数据合法性
        CallResult callResult = tourHouseAdvisorInfoService.verifyRelatedData(tourHouseAdvisorInfo, originalTourHouseAdvisorInfo);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        if (!tourHouseAdvisorInfoService.update(tourHouseAdvisorInfo, originalTourHouseAdvisorInfo)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除房屋顾问信息管理数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourHouseAdvisorInfo.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除房屋顾问信息管理数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourHouseAdvisorInfo.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的房屋顾问信息管理列表。
     *
     * @param tourHouseAdvisorInfoDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourHouseAdvisorInfo.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourHouseAdvisorInfoVo>> list(
            @MyRequestBody TourHouseAdvisorInfoDto tourHouseAdvisorInfoDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourHouseAdvisorInfo tourHouseAdvisorInfoFilter = MyModelUtil.copyTo(tourHouseAdvisorInfoDtoFilter, TourHouseAdvisorInfo.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourHouseAdvisorInfo.class);
        List<TourHouseAdvisorInfo> tourHouseAdvisorInfoList =
                tourHouseAdvisorInfoService.getTourHouseAdvisorInfoListWithRelation(tourHouseAdvisorInfoFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourHouseAdvisorInfoList, TourHouseAdvisorInfoVo.class));
    }

    /**
     * 查看指定房屋顾问信息管理对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourHouseAdvisorInfo.view")
    @GetMapping("/view")
    public ResponseResult<TourHouseAdvisorInfoVo> view(@RequestParam Long id) {
        TourHouseAdvisorInfo tourHouseAdvisorInfo = tourHouseAdvisorInfoService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourHouseAdvisorInfo == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourHouseAdvisorInfoVo tourHouseAdvisorInfoVo = MyModelUtil.copyTo(tourHouseAdvisorInfo, TourHouseAdvisorInfoVo.class);
        return ResponseResult.success(tourHouseAdvisorInfoVo);
    }

    /**
     * 附件文件下载。
     * 这里将图片和其他类型的附件文件放到不同的父目录下，主要为了便于今后图片文件的迁移。
     *
     * @param id 附件所在记录的主键Id。
     * @param fieldName 附件所属的字段名。
     * @param filename  文件名。如果没有提供该参数，就从当前记录的指定字段中读取。
     * @param asImage   下载文件是否为图片。
     * @param response  Http 应答对象。
     */
    @SaCheckPermission("tourHouseAdvisorInfo.view")
    @OperationLog(type = SysOperationLogType.DOWNLOAD, saveResponse = false)
    @GetMapping("/download")
    public void download(
            @RequestParam(required = false) Long id,
            @RequestParam String fieldName,
            @RequestParam String filename,
            @RequestParam Boolean asImage,
            HttpServletResponse response) {
        if (MyCommonUtil.existBlankArgument(fieldName, filename, asImage)) {
            response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
            return;
        }
        // 使用try来捕获异常，是为了保证一旦出现异常可以返回500的错误状态，便于调试。
        // 否则有可能给前端返回的是200的错误码。
        try {
            // 如果请求参数中没有包含主键Id，就判断该文件是否为当前session上传的。
            if (id == null) {
                if (!cacheHelper.existSessionUploadFile(filename)) {
                    ResponseResult.output(HttpServletResponse.SC_FORBIDDEN);
                    return;
                }
            } else {
                TourHouseAdvisorInfo tourHouseAdvisorInfo = tourHouseAdvisorInfoService.getById(id);
                if (tourHouseAdvisorInfo == null) {
                    ResponseResult.output(HttpServletResponse.SC_NOT_FOUND);
                    return;
                }
                String fieldJsonData = (String) ReflectUtil.getFieldValue(tourHouseAdvisorInfo, fieldName);
                if (fieldJsonData == null && !cacheHelper.existSessionUploadFile(filename)) {
                    ResponseResult.output(HttpServletResponse.SC_BAD_REQUEST);
                    return;
                }
                if (!BaseUpDownloader.containFile(fieldJsonData, filename)
                        && !cacheHelper.existSessionUploadFile(filename)) {
                    ResponseResult.output(HttpServletResponse.SC_FORBIDDEN);
                    return;
                }
            }
            UploadStoreInfo storeInfo = MyModelUtil.getUploadStoreInfo(TourHouseAdvisorInfo.class, fieldName);
            if (!storeInfo.isSupportUpload()) {
                ResponseResult.output(HttpServletResponse.SC_NOT_IMPLEMENTED,
                        ResponseResult.error(ErrorCodeEnum.INVALID_UPLOAD_FIELD));
                return;
            }
            BaseUpDownloader upDownloader = upDownloaderFactory.get(storeInfo.getStoreType());
            upDownloader.doDownload(appConfig.getUploadFileBaseDir(),
                    TourHouseAdvisorInfo.class.getSimpleName(), fieldName, filename, asImage, response);
        } catch (Exception e) {
            response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 文件上传操作。
     *
     * @param fieldName  上传文件名。
     * @param asImage    是否作为图片上传。如果是图片，今后下载的时候无需权限验证。否则就是附件上传，下载时需要权限验证。
     * @param uploadFile 上传文件对象。
     */
    @SaCheckPermission("tourHouseAdvisorInfo.view")
    @OperationLog(type = SysOperationLogType.UPLOAD, saveResponse = false)
    @PostMapping("/upload")
    public void upload(
            @RequestParam String fieldName,
            @RequestParam Boolean asImage,
            @RequestParam("uploadFile") MultipartFile uploadFile) throws IOException {
        UploadStoreInfo storeInfo = MyModelUtil.getUploadStoreInfo(TourHouseAdvisorInfo.class, fieldName);
        // 这里就会判断参数中指定的字段，是否支持上传操作。
        if (!storeInfo.isSupportUpload()) {
            ResponseResult.output(HttpServletResponse.SC_FORBIDDEN,
                    ResponseResult.error(ErrorCodeEnum.INVALID_UPLOAD_FIELD));
            return;
        }
        // 根据字段注解中的存储类型，通过工厂方法获取匹配的上传下载实现类，从而解耦。
        BaseUpDownloader upDownloader = upDownloaderFactory.get(storeInfo.getStoreType());
        UploadResponseInfo responseInfo = upDownloader.doUpload(null,
                appConfig.getUploadFileBaseDir(), TourHouseAdvisorInfo.class.getSimpleName(), fieldName, asImage, uploadFile);
        if (Boolean.TRUE.equals(responseInfo.getUploadFailed())) {
            ResponseResult.output(HttpServletResponse.SC_FORBIDDEN,
                    ResponseResult.error(ErrorCodeEnum.UPLOAD_FAILED, responseInfo.getErrorMessage()));
            return;
        }
        cacheHelper.putSessionUploadFile(responseInfo.getFilename());
        ResponseResult.output(ResponseResult.success(responseInfo));
    }

    /**
     * 以字典形式返回全部房屋顾问信息管理数据集合。字典的键值为[id, name]。
     * 白名单接口，登录用户均可访问。
     *
     * @param filter 过滤对象。
     * @return 应答结果对象，包含的数据为 List<Map<String, String>>，map中包含两条记录，key的值分别是id和name，value对应具体数据。
     */
    @GetMapping("/listDict")
    public ResponseResult<List<Map<String, Object>>> listDict(@ParameterObject TourHouseAdvisorInfoDto filter) {
        List<TourHouseAdvisorInfo> resultList =
                tourHouseAdvisorInfoService.getListByFilter(MyModelUtil.copyTo(filter, TourHouseAdvisorInfo.class));
        return ResponseResult.success(
                MyCommonUtil.toDictDataList(resultList, TourHouseAdvisorInfo::getId, TourHouseAdvisorInfo::getName));
    }

    /**
     * 根据字典Id集合，获取查询后的字典数据。
     *
     * @param dictIds 字典Id集合。
     * @return 应答结果对象，包含字典形式的数据集合。
     */
    @GetMapping("/listDictByIds")
    public ResponseResult<List<Map<String, Object>>> listDictByIds(@RequestParam List<Long> dictIds) {
        List<TourHouseAdvisorInfo> resultList = tourHouseAdvisorInfoService.getInList(new HashSet<>(dictIds));
        return ResponseResult.success(
                MyCommonUtil.toDictDataList(resultList, TourHouseAdvisorInfo::getId, TourHouseAdvisorInfo::getName));
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourHouseAdvisorInfo originalTourHouseAdvisorInfo = tourHouseAdvisorInfoService.getById(id);
        if (originalTourHouseAdvisorInfo == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourHouseAdvisorInfoService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
