package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.tourism.common.additional.config.ApplicationConfig;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;

/**
 * 城市定位管理操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "城市定位管理管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourMapData")
public class TourMapDataController {

    @Autowired
    private ApplicationConfig appConfig;
    @Autowired
    private TourMapDataService tourMapDataService;

    /**
     * 新增城市定位管理数据。
     *
     * @param tourMapDataDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"tourMapDataDto.id", "tourMapDataDto.searchString"})
    @SaCheckPermission("tourMapData.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<String> add(@MyRequestBody TourMapDataDto tourMapDataDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourMapDataDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourMapData tourMapData = MyModelUtil.copyTo(tourMapDataDto, TourMapData.class);
        tourMapData = tourMapDataService.saveNew(tourMapData);
        return ResponseResult.success(tourMapData.getId());
    }

    /**
     * 更新城市定位管理数据。
     *
     * @param tourMapDataDto 更新对象。
     * @return 应答结果对象。
     */
    @ApiOperationSupport(ignoreParameters = {"tourMapDataDto.searchString"})
    @SaCheckPermission("tourMapData.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourMapDataDto tourMapDataDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourMapDataDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourMapData tourMapData = MyModelUtil.copyTo(tourMapDataDto, TourMapData.class);
        TourMapData originalTourMapData = tourMapDataService.getById(tourMapData.getId());
        if (originalTourMapData == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourMapDataService.update(tourMapData, originalTourMapData)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除城市定位管理数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourMapData.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody String id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除城市定位管理数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourMapData.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<String> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (String id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的城市定位管理列表。
     *
     * @param tourMapDataDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourMapData.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourMapDataVo>> list(
            @MyRequestBody TourMapDataDto tourMapDataDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourMapData tourMapDataFilter = MyModelUtil.copyTo(tourMapDataDtoFilter, TourMapData.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourMapData.class);
        List<TourMapData> tourMapDataList =
                tourMapDataService.getTourMapDataListWithRelation(tourMapDataFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourMapDataList, TourMapDataVo.class));
    }

    /**
     * 导入主表数据列表。
     *
     * @param importFile 上传的文件，目前仅仅支持xlsx和xls两种格式。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourMapData.import")
    @OperationLog(type = SysOperationLogType.IMPORT)
    @PostMapping("/import")
    public ResponseResult<Void> importBatch(
            @RequestParam Boolean skipHeader,
            @RequestParam("importFile") MultipartFile importFile) throws IOException {
        String filename = ImportUtil.saveImportFile(appConfig.getUploadFileBaseDir(), null, importFile);
        List<ImportUtil.ImportHeaderInfo> headerInfoList = ImportUtil.makeHeaderInfoList(TourMapData.class, null);
        // 下面是导入时需要注意的地方，如果我们缺省生成的代码，与实际情况存在差异，请手动修改。
        // 1. 头信息数据字段，我们只是根据当前的主表实体对象生成了缺省数组，开发者可根据实际情况，对headerInfoList进行修改。
        ImportUtil.ImportHeaderInfo[] headerInfos = headerInfoList.toArray(new ImportUtil.ImportHeaderInfo[]{});
        // 2. 这里需要根据实际情况决定，导入文件中第一行是否为中文头信息，如果是可以跳过。这里我们默认为true。
        // 这里根据自己的实际需求，为doImport的最后一个参数，传递需要进行字典转换的字段集合。
        // 注意，集合中包含需要翻译的Java字段名，如: gradeId。
        Set<String> translatedDictFieldSet = new HashSet<>();
        List<TourMapData> dataList =
                ImportUtil.doImport(headerInfos, skipHeader, filename, TourMapData.class, translatedDictFieldSet);
        tourMapDataService.saveNewBatch(dataList, -1);
        return ResponseResult.success();
    }

    /**
     * 导出符合过滤条件的城市定位管理列表。
     *
     * @param tourMapDataDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @throws IOException 文件读写失败。
     */
    @SaCheckPermission("tourMapData.export")
    @OperationLog(type = SysOperationLogType.EXPORT, saveResponse = false)
    @PostMapping("/export")
    public void export(
            @MyRequestBody TourMapDataDto tourMapDataDtoFilter,
            @MyRequestBody MyOrderParam orderParam) throws IOException {
        TourMapData tourMapDataFilter = MyModelUtil.copyTo(tourMapDataDtoFilter, TourMapData.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourMapData.class);
        List<TourMapData> resultList =
                tourMapDataService.getTourMapDataListWithRelation(tourMapDataFilter, orderBy);
        // 导出文件的标题数组
        // NOTE: 下面的代码中仅仅导出了主表数据，主表聚合计算数据和主表关联字典的数据。
        // 一对一从表数据的导出，可根据需要自行添加。如：headerMap.put("slaveFieldName.xxxField", "标题名称")
        Map<String, String> headerMap = new LinkedHashMap<>(4);
        headerMap.put("id", "id");
        headerMap.put("cityName", "城市名称城市");
        headerMap.put("lon", "经度");
        headerMap.put("lat", "纬度");
        ExportUtil.doExport(resultList, headerMap, "tourMapData.xlsx");
    }

    /**
     * 查看指定城市定位管理对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourMapData.view")
    @GetMapping("/view")
    public ResponseResult<TourMapDataVo> view(@RequestParam String id) {
        TourMapData tourMapData = tourMapDataService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourMapData == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourMapDataVo tourMapDataVo = MyModelUtil.copyTo(tourMapData, TourMapDataVo.class);
        return ResponseResult.success(tourMapDataVo);
    }

    private ResponseResult<Void> doDelete(String id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourMapData originalTourMapData = tourMapDataService.getById(id);
        if (originalTourMapData == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourMapDataService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
