package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 旅游订单操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "旅游订单管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourOrder")
public class TourOrderController {

    @Autowired
    private TourOrderService tourOrderService;

    /**
     * 新增旅游订单数据。
     *
     * @param tourOrderDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {
            "tourOrderDto.id",
            "tourOrderDto.searchString",
            "tourOrderDto.departureDateStart",
            "tourOrderDto.departureDateEnd",
            "tourOrderDto.endDateStart",
            "tourOrderDto.endDateEnd"})
    @SaCheckPermission("tourOrder.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody TourOrderDto tourOrderDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourOrderDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourOrder tourOrder = MyModelUtil.copyTo(tourOrderDto, TourOrder.class);
        // 验证关联Id的数据合法性
        CallResult callResult = tourOrderService.verifyRelatedData(tourOrder, null);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        tourOrder = tourOrderService.saveNew(tourOrder);
        return ResponseResult.success(tourOrder.getId());
    }

    /**
     * 更新旅游订单数据。
     *
     * @param tourOrderDto 更新对象。
     * @return 应答结果对象。
     */
    @ApiOperationSupport(ignoreParameters = {
            "tourOrderDto.searchString",
            "tourOrderDto.departureDateStart",
            "tourOrderDto.departureDateEnd",
            "tourOrderDto.endDateStart",
            "tourOrderDto.endDateEnd"})
    @SaCheckPermission("tourOrder.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourOrderDto tourOrderDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourOrderDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourOrder tourOrder = MyModelUtil.copyTo(tourOrderDto, TourOrder.class);
        TourOrder originalTourOrder = tourOrderService.getById(tourOrder.getId());
        if (originalTourOrder == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        // 验证关联Id的数据合法性
        CallResult callResult = tourOrderService.verifyRelatedData(tourOrder, originalTourOrder);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        if (!tourOrderService.update(tourOrder, originalTourOrder)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除旅游订单数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourOrder.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除旅游订单数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourOrder.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的旅游订单列表。
     *
     * @param tourOrderDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourOrder.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourOrderVo>> list(
            @MyRequestBody TourOrderDto tourOrderDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourOrder tourOrderFilter = MyModelUtil.copyTo(tourOrderDtoFilter, TourOrder.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourOrder.class);
        List<TourOrder> tourOrderList = tourOrderService.getTourOrderListWithRelation(tourOrderFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourOrderList, TourOrderVo.class));
    }

    /**
     * 查看指定旅游订单对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourOrder.view")
    @GetMapping("/view")
    public ResponseResult<TourOrderVo> view(@RequestParam Long id) {
        TourOrder tourOrder = tourOrderService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourOrder == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourOrderVo tourOrderVo = MyModelUtil.copyTo(tourOrder, TourOrderVo.class);
        return ResponseResult.success(tourOrderVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourOrder originalTourOrder = tourOrderService.getById(id);
        if (originalTourOrder == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourOrderService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
