package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 开团申请列表操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "开团申请列表管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourProjectGroupPurchaseApply")
public class TourProjectGroupPurchaseApplyController {

    @Autowired
    private TourProjectGroupPurchaseApplyService tourProjectGroupPurchaseApplyService;

    /**
     * 新增开团申请列表数据。
     *
     * @param tourProjectGroupPurchaseApplyDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {
            "tourProjectGroupPurchaseApplyDto.id",
            "tourProjectGroupPurchaseApplyDto.applyTimeStart",
            "tourProjectGroupPurchaseApplyDto.applyTimeEnd"})
    @SaCheckPermission("tourProjectGroupPurchaseApply.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody TourProjectGroupPurchaseApplyDto tourProjectGroupPurchaseApplyDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourProjectGroupPurchaseApplyDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourProjectGroupPurchaseApply tourProjectGroupPurchaseApply = MyModelUtil.copyTo(tourProjectGroupPurchaseApplyDto, TourProjectGroupPurchaseApply.class);
        // 验证关联Id的数据合法性
        CallResult callResult = tourProjectGroupPurchaseApplyService.verifyRelatedData(tourProjectGroupPurchaseApply, null);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        tourProjectGroupPurchaseApply = tourProjectGroupPurchaseApplyService.saveNew(tourProjectGroupPurchaseApply);
        return ResponseResult.success(tourProjectGroupPurchaseApply.getId());
    }

    /**
     * 更新开团申请列表数据。
     *
     * @param tourProjectGroupPurchaseApplyDto 更新对象。
     * @return 应答结果对象。
     */
    @ApiOperationSupport(ignoreParameters = {
            "tourProjectGroupPurchaseApplyDto.applyTimeStart",
            "tourProjectGroupPurchaseApplyDto.applyTimeEnd"})
    @SaCheckPermission("tourProjectGroupPurchaseApply.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourProjectGroupPurchaseApplyDto tourProjectGroupPurchaseApplyDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourProjectGroupPurchaseApplyDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourProjectGroupPurchaseApply tourProjectGroupPurchaseApply = MyModelUtil.copyTo(tourProjectGroupPurchaseApplyDto, TourProjectGroupPurchaseApply.class);
        TourProjectGroupPurchaseApply originalTourProjectGroupPurchaseApply = tourProjectGroupPurchaseApplyService.getById(tourProjectGroupPurchaseApply.getId());
        if (originalTourProjectGroupPurchaseApply == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        // 验证关联Id的数据合法性
        CallResult callResult = tourProjectGroupPurchaseApplyService.verifyRelatedData(tourProjectGroupPurchaseApply, originalTourProjectGroupPurchaseApply);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        if (!tourProjectGroupPurchaseApplyService.update(tourProjectGroupPurchaseApply, originalTourProjectGroupPurchaseApply)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除开团申请列表数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourProjectGroupPurchaseApply.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除开团申请列表数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourProjectGroupPurchaseApply.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的开团申请列表列表。
     *
     * @param tourProjectGroupPurchaseApplyDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourProjectGroupPurchaseApply.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourProjectGroupPurchaseApplyVo>> list(
            @MyRequestBody TourProjectGroupPurchaseApplyDto tourProjectGroupPurchaseApplyDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourProjectGroupPurchaseApply tourProjectGroupPurchaseApplyFilter = MyModelUtil.copyTo(tourProjectGroupPurchaseApplyDtoFilter, TourProjectGroupPurchaseApply.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourProjectGroupPurchaseApply.class);
        List<TourProjectGroupPurchaseApply> tourProjectGroupPurchaseApplyList =
                tourProjectGroupPurchaseApplyService.getTourProjectGroupPurchaseApplyListWithRelation(tourProjectGroupPurchaseApplyFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourProjectGroupPurchaseApplyList, TourProjectGroupPurchaseApplyVo.class));
    }

    /**
     * 查看指定开团申请列表对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourProjectGroupPurchaseApply.view")
    @GetMapping("/view")
    public ResponseResult<TourProjectGroupPurchaseApplyVo> view(@RequestParam Long id) {
        TourProjectGroupPurchaseApply tourProjectGroupPurchaseApply = tourProjectGroupPurchaseApplyService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourProjectGroupPurchaseApply == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourProjectGroupPurchaseApplyVo tourProjectGroupPurchaseApplyVo = MyModelUtil.copyTo(tourProjectGroupPurchaseApply, TourProjectGroupPurchaseApplyVo.class);
        return ResponseResult.success(tourProjectGroupPurchaseApplyVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourProjectGroupPurchaseApply originalTourProjectGroupPurchaseApply = tourProjectGroupPurchaseApplyService.getById(id);
        if (originalTourProjectGroupPurchaseApply == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourProjectGroupPurchaseApplyService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
