package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.util.ObjectUtil;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 游记评论操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "游记评论管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourTravelNotesComment")
public class TourTravelNotesCommentController {

    @Autowired
    private TourTravelNotesCommentService tourTravelNotesCommentService;

    /**
     * 新增游记评论数据。
     *
     * @param tourTravelNotesCommentDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"tourTravelNotesCommentDto.id"})
    @SaCheckPermission("tourTravelNotesComment.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<String> add(@MyRequestBody TourTravelNotesCommentDto tourTravelNotesCommentDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourTravelNotesCommentDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourTravelNotesComment tourTravelNotesComment = MyModelUtil.copyTo(tourTravelNotesCommentDto, TourTravelNotesComment.class);
        // 验证父Id的数据合法性
        TourTravelNotesComment parentTourTravelNotesComment;
        if (MyCommonUtil.isNotBlankOrNull(tourTravelNotesComment.getParentId())) {
            parentTourTravelNotesComment = tourTravelNotesCommentService.getById(tourTravelNotesComment.getParentId());
            if (parentTourTravelNotesComment == null) {
                errorMessage = "数据验证失败，关联的父节点并不存在，请刷新后重试！";
                return ResponseResult.error(ErrorCodeEnum.DATA_PARENT_ID_NOT_EXIST, errorMessage);
            }
        }
        tourTravelNotesComment = tourTravelNotesCommentService.saveNew(tourTravelNotesComment);
        return ResponseResult.success(tourTravelNotesComment.getId());
    }

    /**
     * 更新游记评论数据。
     *
     * @param tourTravelNotesCommentDto 更新对象。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourTravelNotesComment.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourTravelNotesCommentDto tourTravelNotesCommentDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourTravelNotesCommentDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourTravelNotesComment tourTravelNotesComment = MyModelUtil.copyTo(tourTravelNotesCommentDto, TourTravelNotesComment.class);
        TourTravelNotesComment originalTourTravelNotesComment = tourTravelNotesCommentService.getById(tourTravelNotesComment.getId());
        if (originalTourTravelNotesComment == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        // 验证父Id的数据合法性
        if (MyCommonUtil.isNotBlankOrNull(tourTravelNotesComment.getParentId())
                && ObjectUtil.notEqual(tourTravelNotesComment.getParentId(), originalTourTravelNotesComment.getParentId())) {
            TourTravelNotesComment parentTourTravelNotesComment = tourTravelNotesCommentService.getById(tourTravelNotesComment.getParentId());
            if (parentTourTravelNotesComment == null) {
                // NOTE: 修改下面方括号中的话述
                errorMessage = "数据验证失败，关联的 [父节点] 并不存在，请刷新后重试！";
                return ResponseResult.error(ErrorCodeEnum.DATA_PARENT_ID_NOT_EXIST, errorMessage);
            }
        }
        if (!tourTravelNotesCommentService.update(tourTravelNotesComment, originalTourTravelNotesComment)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除游记评论数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourTravelNotesComment.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除游记评论数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourTravelNotesComment.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的游记评论列表。
     *
     * @param tourTravelNotesCommentDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourTravelNotesComment.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourTravelNotesCommentVo>> list(
            @MyRequestBody TourTravelNotesCommentDto tourTravelNotesCommentDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourTravelNotesComment tourTravelNotesCommentFilter = MyModelUtil.copyTo(tourTravelNotesCommentDtoFilter, TourTravelNotesComment.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourTravelNotesComment.class);
        List<TourTravelNotesComment> tourTravelNotesCommentList =
                tourTravelNotesCommentService.getTourTravelNotesCommentListWithRelation(tourTravelNotesCommentFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourTravelNotesCommentList, TourTravelNotesCommentVo.class));
    }

    /**
     * 查看指定游记评论对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourTravelNotesComment.view")
    @GetMapping("/view")
    public ResponseResult<TourTravelNotesCommentVo> view(@RequestParam Long id) {
        TourTravelNotesComment tourTravelNotesComment = tourTravelNotesCommentService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourTravelNotesComment == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourTravelNotesCommentVo tourTravelNotesCommentVo = MyModelUtil.copyTo(tourTravelNotesComment, TourTravelNotesCommentVo.class);
        return ResponseResult.success(tourTravelNotesCommentVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourTravelNotesComment originalTourTravelNotesComment = tourTravelNotesCommentService.getById(id);
        if (originalTourTravelNotesComment == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (tourTravelNotesCommentService.hasChildren(id)) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象存在子对象] ，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.HAS_CHILDREN_DATA, errorMessage);
        }
        if (!tourTravelNotesCommentService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
