package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.alibaba.fastjson.JSONObject;
import cn.hutool.core.util.ReflectUtil;
import com.tourism.common.additional.config.ApplicationConfig;
import com.tourism.common.core.upload.BaseUpDownloader;
import com.tourism.common.core.upload.UpDownloaderFactory;
import com.tourism.common.core.upload.UploadResponseInfo;
import com.tourism.common.core.upload.UploadStoreInfo;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.tourism.common.redis.cache.SessionCacheHelper;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import jakarta.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.*;

/**
 * 签证信息管理操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "签证信息管理管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourVisa")
public class TourVisaController {

    @Autowired
    private ApplicationConfig appConfig;
    @Autowired
    private SessionCacheHelper cacheHelper;
    @Autowired
    private UpDownloaderFactory upDownloaderFactory;
    @Autowired
    private TourVisaService tourVisaService;

    /**
     * 新增签证信息管理数据，及其关联的从表数据。
     *
     * @param tourVisaDto 新增主表对象。
     * @param visaMetarialsListDtoList 一对多签证基础资料管理从表列表。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"tourVisaDto.id", "tourVisaDto.searchString"})
    @SaCheckPermission("tourVisa.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(
            @MyRequestBody TourVisaDto tourVisaDto,
            @MyRequestBody List<VisaMetarialsListDto> visaMetarialsListDtoList) {
        ResponseResult<Tuple2<TourVisa, JSONObject>> verifyResult =
                this.doBusinessDataVerifyAndConvert(tourVisaDto, false, visaMetarialsListDtoList);
        if (!verifyResult.isSuccess()) {
            return ResponseResult.errorFrom(verifyResult);
        }
        Tuple2<TourVisa, JSONObject> bizData = verifyResult.getData();
        TourVisa tourVisa = bizData.getFirst();
        tourVisa = tourVisaService.saveNewWithRelation(tourVisa, bizData.getSecond());
        return ResponseResult.success(tourVisa.getId());
    }

    /**
     * 修改签证信息管理数据，及其关联的从表数据。
     *
     * @param tourVisaDto 修改后的对象。
     * @param visaMetarialsListDtoList 一对多签证基础资料管理从表列表。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"tourVisaDto.id", "tourVisaDto.searchString"})
    @SaCheckPermission("tourVisa.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Long> update(
            @MyRequestBody TourVisaDto tourVisaDto,
            @MyRequestBody List<VisaMetarialsListDto> visaMetarialsListDtoList) {
        String errorMessage;
        ResponseResult<Tuple2<TourVisa, JSONObject>> verifyResult =
                this.doBusinessDataVerifyAndConvert(tourVisaDto, true, visaMetarialsListDtoList);
        if (!verifyResult.isSuccess()) {
            return ResponseResult.errorFrom(verifyResult);
        }
        Tuple2<TourVisa, JSONObject> bizData = verifyResult.getData();
        TourVisa originalTourVisa = bizData.getSecond().getObject("originalData", TourVisa.class);
        TourVisa tourVisa = bizData.getFirst();
        if (!tourVisaService.updateWithRelation(tourVisa, originalTourVisa, bizData.getSecond())) {
            errorMessage = "数据验证失败，[TourVisa] 数据不存在!";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success(tourVisa.getId());
    }

    /**
     * 删除签证信息管理数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourVisa.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除签证信息管理数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourVisa.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的签证信息管理列表。
     *
     * @param tourVisaDtoFilter 过滤对象。
     * @param visaMetarialsListDtoFilter 一对多从表过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourVisa.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourVisaVo>> list(
            @MyRequestBody TourVisaDto tourVisaDtoFilter,
            @MyRequestBody VisaMetarialsListDto visaMetarialsListDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourVisa tourVisaFilter = MyModelUtil.copyTo(tourVisaDtoFilter, TourVisa.class);
        VisaMetarialsList visaMetarialsListFilter = MyModelUtil.copyTo(visaMetarialsListDtoFilter, VisaMetarialsList.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourVisa.class);
        List<TourVisa> tourVisaList =
                tourVisaService.getTourVisaListWithRelation(tourVisaFilter, visaMetarialsListFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourVisaList, TourVisaVo.class));
    }

    /**
     * 导入主表数据列表。
     *
     * @param importFile 上传的文件，目前仅仅支持xlsx和xls两种格式。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourVisa.import")
    @OperationLog(type = SysOperationLogType.IMPORT)
    @PostMapping("/import")
    public ResponseResult<Void> importBatch(
            @RequestParam Boolean skipHeader,
            @RequestParam("importFile") MultipartFile importFile) throws IOException {
        String filename = ImportUtil.saveImportFile(appConfig.getUploadFileBaseDir(), null, importFile);
        // 这里可以指定需要忽略导入的字段集合。如创建时间、创建人、更新时间、更新人、主键Id和逻辑删除，
        // 以及一些存在缺省值且无需导入的字段。其中主键字段和逻辑删除字段不需要在这里设置，批量插入逻辑会自动处理的。
        Set<String> ignoreFieldSet = new HashSet<>();
        ignoreFieldSet.add("createUserId");
        ignoreFieldSet.add("createTime");
        ignoreFieldSet.add("updateUserId");
        ignoreFieldSet.add("updateTime");
        List<ImportUtil.ImportHeaderInfo> headerInfoList = ImportUtil.makeHeaderInfoList(TourVisa.class, ignoreFieldSet);
        // 下面是导入时需要注意的地方，如果我们缺省生成的代码，与实际情况存在差异，请手动修改。
        // 1. 头信息数据字段，我们只是根据当前的主表实体对象生成了缺省数组，开发者可根据实际情况，对headerInfoList进行修改。
        ImportUtil.ImportHeaderInfo[] headerInfos = headerInfoList.toArray(new ImportUtil.ImportHeaderInfo[]{});
        // 2. 这里需要根据实际情况决定，导入文件中第一行是否为中文头信息，如果是可以跳过。这里我们默认为true。
        // 这里根据自己的实际需求，为doImport的最后一个参数，传递需要进行字典转换的字段集合。
        // 注意，集合中包含需要翻译的Java字段名，如: gradeId。
        Set<String> translatedDictFieldSet = new HashSet<>();
        List<TourVisa> dataList =
                ImportUtil.doImport(headerInfos, skipHeader, filename, TourVisa.class, translatedDictFieldSet);
        CallResult result = tourVisaService.verifyImportList(dataList, translatedDictFieldSet);
        if (!result.isSuccess()) {
            // result中返回了具体的验证失败对象，如果需要返回更加详细的错误，可根据实际情况手动修改。
            return ResponseResult.errorFrom(result);
        }
        tourVisaService.saveNewBatch(dataList, -1);
        return ResponseResult.success();
    }

    /**
     * 导出符合过滤条件的签证信息管理列表。
     *
     * @param tourVisaDtoFilter 过滤对象。
     * @param visaMetarialsListDtoFilter 一对多从表过滤对象。
     * @param orderParam 排序参数。
     * @throws IOException 文件读写失败。
     */
    @SaCheckPermission("tourVisa.export")
    @OperationLog(type = SysOperationLogType.EXPORT, saveResponse = false)
    @PostMapping("/export")
    public void export(
            @MyRequestBody TourVisaDto tourVisaDtoFilter,
            @MyRequestBody VisaMetarialsListDto visaMetarialsListDtoFilter,
            @MyRequestBody MyOrderParam orderParam) throws IOException {
        TourVisa tourVisaFilter = MyModelUtil.copyTo(tourVisaDtoFilter, TourVisa.class);
        VisaMetarialsList visaMetarialsListFilter = MyModelUtil.copyTo(visaMetarialsListDtoFilter, VisaMetarialsList.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourVisa.class);
        List<TourVisa> resultList =
                tourVisaService.getTourVisaListWithRelation(tourVisaFilter, visaMetarialsListFilter, orderBy);
        // 导出文件的标题数组
        // NOTE: 下面的代码中仅仅导出了主表数据，主表聚合计算数据和主表关联字典的数据。
        // 一对一从表数据的导出，可根据需要自行添加。如：headerMap.put("slaveFieldName.xxxField", "标题名称")
        Map<String, String> headerMap = new LinkedHashMap<>(23);
        headerMap.put("idDictMap.name", "签证id");
        headerMap.put("img", "图片地址");
        headerMap.put("visaTitle", "名称");
        headerMap.put("visaTypeIdDictMap.name", "类型id");
        headerMap.put("countryCodeIdDictMap.name", "国家编码");
        headerMap.put("embassyName", "使馆名称");
        headerMap.put("embassyAddress", "使馆地址");
        headerMap.put("interviewFlagDictMap.name", "是否面签 0 是 1否");
        headerMap.put("validity", "有效期（日）");
        headerMap.put("idleDays", "停留天数");
        headerMap.put("processingDays", "预计办理天数");
        headerMap.put("floorPrice", "预计价格");
        headerMap.put("acceptanceRange", "受理范围");
        headerMap.put("entriesNum", "入境次数");
        headerMap.put("dataText", "预定须知");
        headerMap.put("issueVisaCodeDictMap.name", "出签地编码");
        headerMap.put("deleteFlag", "逻辑删除 1正常 -1已删除");
        headerMap.put("createUserId", "创建人id");
        headerMap.put("createTime", "创建时间");
        headerMap.put("updateUserId", "更新人id");
        headerMap.put("updateTime", "更新时间");
        headerMap.put("counselorIdDictMap.name", "负责顾问");
        headerMap.put("hotValueDictMap.name", "热门数据");
        ExportUtil.doExport(resultList, headerMap, "tourVisa.xlsx");
    }

    /**
     * 查看指定签证信息管理对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourVisa.view")
    @GetMapping("/view")
    public ResponseResult<TourVisaVo> view(@RequestParam Long id) {
        TourVisa tourVisa = tourVisaService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourVisa == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourVisaVo tourVisaVo = MyModelUtil.copyTo(tourVisa, TourVisaVo.class);
        return ResponseResult.success(tourVisaVo);
    }

    /**
     * 附件文件下载。
     * 这里将图片和其他类型的附件文件放到不同的父目录下，主要为了便于今后图片文件的迁移。
     *
     * @param id 附件所在记录的主键Id。
     * @param fieldName 附件所属的字段名。
     * @param filename  文件名。如果没有提供该参数，就从当前记录的指定字段中读取。
     * @param asImage   下载文件是否为图片。
     * @param response  Http 应答对象。
     */
    @SaCheckPermission("tourVisa.view")
    @OperationLog(type = SysOperationLogType.DOWNLOAD, saveResponse = false)
    @GetMapping("/download")
    public void download(
            @RequestParam(required = false) Long id,
            @RequestParam String fieldName,
            @RequestParam String filename,
            @RequestParam Boolean asImage,
            HttpServletResponse response) {
        if (MyCommonUtil.existBlankArgument(fieldName, filename, asImage)) {
            response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
            return;
        }
        // 使用try来捕获异常，是为了保证一旦出现异常可以返回500的错误状态，便于调试。
        // 否则有可能给前端返回的是200的错误码。
        try {
            // 如果请求参数中没有包含主键Id，就判断该文件是否为当前session上传的。
            if (id == null) {
                if (!cacheHelper.existSessionUploadFile(filename)) {
                    ResponseResult.output(HttpServletResponse.SC_FORBIDDEN);
                    return;
                }
            } else {
                TourVisa tourVisa = tourVisaService.getById(id);
                if (tourVisa == null) {
                    ResponseResult.output(HttpServletResponse.SC_NOT_FOUND);
                    return;
                }
                String fieldJsonData = (String) ReflectUtil.getFieldValue(tourVisa, fieldName);
                if (fieldJsonData == null && !cacheHelper.existSessionUploadFile(filename)) {
                    ResponseResult.output(HttpServletResponse.SC_BAD_REQUEST);
                    return;
                }
                if (!BaseUpDownloader.containFile(fieldJsonData, filename)
                        && !cacheHelper.existSessionUploadFile(filename)) {
                    ResponseResult.output(HttpServletResponse.SC_FORBIDDEN);
                    return;
                }
            }
            UploadStoreInfo storeInfo = MyModelUtil.getUploadStoreInfo(TourVisa.class, fieldName);
            if (!storeInfo.isSupportUpload()) {
                ResponseResult.output(HttpServletResponse.SC_NOT_IMPLEMENTED,
                        ResponseResult.error(ErrorCodeEnum.INVALID_UPLOAD_FIELD));
                return;
            }
            BaseUpDownloader upDownloader = upDownloaderFactory.get(storeInfo.getStoreType());
            upDownloader.doDownload(appConfig.getUploadFileBaseDir(),
                    TourVisa.class.getSimpleName(), fieldName, filename, asImage, response);
        } catch (Exception e) {
            response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 文件上传操作。
     *
     * @param fieldName  上传文件名。
     * @param asImage    是否作为图片上传。如果是图片，今后下载的时候无需权限验证。否则就是附件上传，下载时需要权限验证。
     * @param uploadFile 上传文件对象。
     */
    @SaCheckPermission("tourVisa.view")
    @OperationLog(type = SysOperationLogType.UPLOAD, saveResponse = false)
    @PostMapping("/upload")
    public void upload(
            @RequestParam String fieldName,
            @RequestParam Boolean asImage,
            @RequestParam("uploadFile") MultipartFile uploadFile) throws IOException {
        UploadStoreInfo storeInfo = MyModelUtil.getUploadStoreInfo(TourVisa.class, fieldName);
        // 这里就会判断参数中指定的字段，是否支持上传操作。
        if (!storeInfo.isSupportUpload()) {
            ResponseResult.output(HttpServletResponse.SC_FORBIDDEN,
                    ResponseResult.error(ErrorCodeEnum.INVALID_UPLOAD_FIELD));
            return;
        }
        // 根据字段注解中的存储类型，通过工厂方法获取匹配的上传下载实现类，从而解耦。
        BaseUpDownloader upDownloader = upDownloaderFactory.get(storeInfo.getStoreType());
        UploadResponseInfo responseInfo = upDownloader.doUpload(null,
                appConfig.getUploadFileBaseDir(), TourVisa.class.getSimpleName(), fieldName, asImage, uploadFile);
        if (Boolean.TRUE.equals(responseInfo.getUploadFailed())) {
            ResponseResult.output(HttpServletResponse.SC_FORBIDDEN,
                    ResponseResult.error(ErrorCodeEnum.UPLOAD_FAILED, responseInfo.getErrorMessage()));
            return;
        }
        cacheHelper.putSessionUploadFile(responseInfo.getFilename());
        ResponseResult.output(ResponseResult.success(responseInfo));
    }

    /**
     * 以字典形式返回全部签证信息管理数据集合。字典的键值为[id, visaTitle]。
     * 白名单接口，登录用户均可访问。
     *
     * @param filter 过滤对象。
     * @return 应答结果对象，包含的数据为 List<Map<String, String>>，map中包含两条记录，key的值分别是id和name，value对应具体数据。
     */
    @GetMapping("/listDict")
    public ResponseResult<List<Map<String, Object>>> listDict(@ParameterObject TourVisaDto filter) {
        List<TourVisa> resultList =
                tourVisaService.getListByFilter(MyModelUtil.copyTo(filter, TourVisa.class));
        return ResponseResult.success(
                MyCommonUtil.toDictDataList(resultList, TourVisa::getId, TourVisa::getVisaTitle));
    }

    /**
     * 根据字典Id集合，获取查询后的字典数据。
     *
     * @param dictIds 字典Id集合。
     * @return 应答结果对象，包含字典形式的数据集合。
     */
    @GetMapping("/listDictByIds")
    public ResponseResult<List<Map<String, Object>>> listDictByIds(@RequestParam List<Long> dictIds) {
        List<TourVisa> resultList = tourVisaService.getInList(new HashSet<>(dictIds));
        return ResponseResult.success(
                MyCommonUtil.toDictDataList(resultList, TourVisa::getId, TourVisa::getVisaTitle));
    }

    private ResponseResult<Tuple2<TourVisa, JSONObject>> doBusinessDataVerifyAndConvert(
            TourVisaDto tourVisaDto,
            boolean forUpdate,
            List<VisaMetarialsListDto> visaMetarialsListDtoList) {
        ErrorCodeEnum errorCode = ErrorCodeEnum.DATA_VALIDATED_FAILED;
        String errorMessage = MyCommonUtil.getModelValidationError(tourVisaDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(errorCode, errorMessage);
        }
        errorMessage = MyCommonUtil.getModelValidationError(visaMetarialsListDtoList);
        if (errorMessage != null) {
            return ResponseResult.error(errorCode, "参数 [visaMetarialsListDtoList] " + errorMessage);
        }
        // 全部关联从表数据的验证和转换
        JSONObject relationData = new JSONObject();
        CallResult verifyResult;
        // 下面是输入参数中，主表关联数据的验证。
        TourVisa tourVisa = MyModelUtil.copyTo(tourVisaDto, TourVisa.class);
        TourVisa originalData = null;
        if (forUpdate && tourVisa != null) {
            originalData = tourVisaService.getById(tourVisa.getId());
            if (originalData == null) {
                return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
            }
            relationData.put("originalData", originalData);
        }
        verifyResult = tourVisaService.verifyRelatedData(tourVisa, originalData);
        if (!verifyResult.isSuccess()) {
            return ResponseResult.errorFrom(verifyResult);
        }
        // 处理主表的一对多关联 [VisaMetarialsList]
        List<VisaMetarialsList> visaMetarialsListList =
                MyModelUtil.copyCollectionTo(visaMetarialsListDtoList, VisaMetarialsList.class);
        relationData.put("visaMetarialsListList", visaMetarialsListList);
        return ResponseResult.success(new Tuple2<>(tourVisa, relationData));
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourVisa originalTourVisa = tourVisaService.getById(id);
        if (originalTourVisa == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourVisaService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
