package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.util.ReflectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.tourism.common.additional.config.ApplicationConfig;
import com.tourism.common.core.upload.BaseUpDownloader;
import com.tourism.common.core.upload.UpDownloaderFactory;
import com.tourism.common.core.upload.UploadResponseInfo;
import com.tourism.common.core.upload.UploadStoreInfo;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.tourism.common.redis.cache.SessionCacheHelper;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import jakarta.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.*;

/**
 * 签证顾问信息操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "签证顾问信息管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourVisaCounselor")
public class TourVisaCounselorController {

    @Autowired
    private ApplicationConfig appConfig;
    @Autowired
    private SessionCacheHelper cacheHelper;
    @Autowired
    private UpDownloaderFactory upDownloaderFactory;
    @Autowired
    private TourVisaCounselorService tourVisaCounselorService;

    /**
     * 新增签证顾问信息数据。
     *
     * @param tourVisaCounselorDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"tourVisaCounselorDto.id", "tourVisaCounselorDto.searchString"})
    @SaCheckPermission("tourVisaCounselor.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody TourVisaCounselorDto tourVisaCounselorDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourVisaCounselorDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        if (tourVisaCounselorDto.getIndexFlag() == 1){
            List<TourVisaCounselor> list = tourVisaCounselorService.list(new LambdaQueryWrapper<TourVisaCounselor>().eq(TourVisaCounselor::getIndexFlag, 1));
            if (list != null && !list.isEmpty()){
                return ResponseResult.error(ErrorCodeEnum.NO_ERROR,"只能选择一个作为列表展示顾问");
            }
        }
        TourVisaCounselor tourVisaCounselor = MyModelUtil.copyTo(tourVisaCounselorDto, TourVisaCounselor.class);
        tourVisaCounselor = tourVisaCounselorService.saveNew(tourVisaCounselor);
        return ResponseResult.success(tourVisaCounselor.getId());
    }

    /**
     * 更新签证顾问信息数据。
     *
     * @param tourVisaCounselorDto 更新对象。
     * @return 应答结果对象。
     */
    @ApiOperationSupport(ignoreParameters = {"tourVisaCounselorDto.searchString"})
    @SaCheckPermission("tourVisaCounselor.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourVisaCounselorDto tourVisaCounselorDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourVisaCounselorDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        if (tourVisaCounselorDto.getIndexFlag() == 1){
            TourVisaCounselor one = tourVisaCounselorService.getOne(new LambdaQueryWrapper<TourVisaCounselor>().eq(TourVisaCounselor::getIndexFlag, 1));
            if (one != null && !one.getId().toString().equals(tourVisaCounselorDto.getId().toString())){
                return ResponseResult.error(ErrorCodeEnum.NO_ERROR,"请取消上一个显示顾问后再设置显示顾问");

            }
        }
        TourVisaCounselor tourVisaCounselor = MyModelUtil.copyTo(tourVisaCounselorDto, TourVisaCounselor.class);
        TourVisaCounselor originalTourVisaCounselor = tourVisaCounselorService.getById(tourVisaCounselor.getId());
        if (originalTourVisaCounselor == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourVisaCounselorService.update(tourVisaCounselor, originalTourVisaCounselor)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除签证顾问信息数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourVisaCounselor.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除签证顾问信息数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourVisaCounselor.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的签证顾问信息列表。
     *
     * @param tourVisaCounselorDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourVisaCounselor.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourVisaCounselorVo>> list(
            @MyRequestBody TourVisaCounselorDto tourVisaCounselorDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourVisaCounselor tourVisaCounselorFilter = MyModelUtil.copyTo(tourVisaCounselorDtoFilter, TourVisaCounselor.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourVisaCounselor.class);
        List<TourVisaCounselor> tourVisaCounselorList =
                tourVisaCounselorService.getTourVisaCounselorListWithRelation(tourVisaCounselorFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourVisaCounselorList, TourVisaCounselorVo.class));
    }

    /**
     * 导入主表数据列表。
     *
     * @param importFile 上传的文件，目前仅仅支持xlsx和xls两种格式。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourVisaCounselor.import")
    @OperationLog(type = SysOperationLogType.IMPORT)
    @PostMapping("/import")
    public ResponseResult<Void> importBatch(
            @RequestParam Boolean skipHeader,
            @RequestParam("importFile") MultipartFile importFile) throws IOException {
        String filename = ImportUtil.saveImportFile(appConfig.getUploadFileBaseDir(), null, importFile);
        // 这里可以指定需要忽略导入的字段集合。如创建时间、创建人、更新时间、更新人、主键Id和逻辑删除，
        // 以及一些存在缺省值且无需导入的字段。其中主键字段和逻辑删除字段不需要在这里设置，批量插入逻辑会自动处理的。
        Set<String> ignoreFieldSet = new HashSet<>();
        ignoreFieldSet.add("createUserId");
        ignoreFieldSet.add("createTime");
        ignoreFieldSet.add("updateUserId");
        ignoreFieldSet.add("updateTime");
        List<ImportUtil.ImportHeaderInfo> headerInfoList = ImportUtil.makeHeaderInfoList(TourVisaCounselor.class, ignoreFieldSet);
        // 下面是导入时需要注意的地方，如果我们缺省生成的代码，与实际情况存在差异，请手动修改。
        // 1. 头信息数据字段，我们只是根据当前的主表实体对象生成了缺省数组，开发者可根据实际情况，对headerInfoList进行修改。
        ImportUtil.ImportHeaderInfo[] headerInfos = headerInfoList.toArray(new ImportUtil.ImportHeaderInfo[]{});
        // 2. 这里需要根据实际情况决定，导入文件中第一行是否为中文头信息，如果是可以跳过。这里我们默认为true。
        // 这里根据自己的实际需求，为doImport的最后一个参数，传递需要进行字典转换的字段集合。
        // 注意，集合中包含需要翻译的Java字段名，如: gradeId。
        Set<String> translatedDictFieldSet = new HashSet<>();
        List<TourVisaCounselor> dataList =
                ImportUtil.doImport(headerInfos, skipHeader, filename, TourVisaCounselor.class, translatedDictFieldSet);
        CallResult result = tourVisaCounselorService.verifyImportList(dataList, translatedDictFieldSet);
        if (!result.isSuccess()) {
            // result中返回了具体的验证失败对象，如果需要返回更加详细的错误，可根据实际情况手动修改。
            return ResponseResult.errorFrom(result);
        }
        tourVisaCounselorService.saveNewBatch(dataList, -1);
        return ResponseResult.success();
    }

    /**
     * 导出符合过滤条件的签证顾问信息列表。
     *
     * @param tourVisaCounselorDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @throws IOException 文件读写失败。
     */
    @SaCheckPermission("tourVisaCounselor.export")
    @OperationLog(type = SysOperationLogType.EXPORT, saveResponse = false)
    @PostMapping("/export")
    public void export(
            @MyRequestBody TourVisaCounselorDto tourVisaCounselorDtoFilter,
            @MyRequestBody MyOrderParam orderParam) throws IOException {
        TourVisaCounselor tourVisaCounselorFilter = MyModelUtil.copyTo(tourVisaCounselorDtoFilter, TourVisaCounselor.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourVisaCounselor.class);
        List<TourVisaCounselor> resultList =
                tourVisaCounselorService.getTourVisaCounselorListWithRelation(tourVisaCounselorFilter, orderBy);
        // 导出文件的标题数组
        // NOTE: 下面的代码中仅仅导出了主表数据，主表聚合计算数据和主表关联字典的数据。
        // 一对一从表数据的导出，可根据需要自行添加。如：headerMap.put("slaveFieldName.xxxField", "标题名称")
        Map<String, String> headerMap = new LinkedHashMap<>(14);
        headerMap.put("id", "顾问id");
        headerMap.put("counselorAvatar", "顾问头像");
        headerMap.put("counselorPhone", "顾问电话");
        headerMap.put("counselorWechat", "顾问微信");
        headerMap.put("counselorName", "顾问姓名");
        headerMap.put("deleteFlag", "逻辑删除 1正常 -1已删除");
        headerMap.put("createUserId", "创建人id");
        headerMap.put("createTime", "创建时间");
        headerMap.put("updateUserId", "更新人id");
        headerMap.put("updateTime", "更新时间");
        headerMap.put("levelDictMap.name", "级别");
        headerMap.put("start", "上班时间");
        headerMap.put("end", "下班时间");
        headerMap.put("indexFlagDictMap.name", "首页显示顾问");
        ExportUtil.doExport(resultList, headerMap, "tourVisaCounselor.xlsx");
    }

    /**
     * 查看指定签证顾问信息对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourVisaCounselor.view")
    @GetMapping("/view")
    public ResponseResult<TourVisaCounselorVo> view(@RequestParam Long id) {
        TourVisaCounselor tourVisaCounselor = tourVisaCounselorService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourVisaCounselor == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourVisaCounselorVo tourVisaCounselorVo = MyModelUtil.copyTo(tourVisaCounselor, TourVisaCounselorVo.class);
        return ResponseResult.success(tourVisaCounselorVo);
    }

    /**
     * 附件文件下载。
     * 这里将图片和其他类型的附件文件放到不同的父目录下，主要为了便于今后图片文件的迁移。
     *
     * @param id 附件所在记录的主键Id。
     * @param fieldName 附件所属的字段名。
     * @param filename  文件名。如果没有提供该参数，就从当前记录的指定字段中读取。
     * @param asImage   下载文件是否为图片。
     * @param response  Http 应答对象。
     */
    @SaCheckPermission("tourVisaCounselor.view")
    @OperationLog(type = SysOperationLogType.DOWNLOAD, saveResponse = false)
    @GetMapping("/download")
    public void download(
            @RequestParam(required = false) Long id,
            @RequestParam String fieldName,
            @RequestParam String filename,
            @RequestParam Boolean asImage,
            HttpServletResponse response) {
        if (MyCommonUtil.existBlankArgument(fieldName, filename, asImage)) {
            response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
            return;
        }
        // 使用try来捕获异常，是为了保证一旦出现异常可以返回500的错误状态，便于调试。
        // 否则有可能给前端返回的是200的错误码。
        try {
            // 如果请求参数中没有包含主键Id，就判断该文件是否为当前session上传的。
            if (id == null) {
                if (!cacheHelper.existSessionUploadFile(filename)) {
                    ResponseResult.output(HttpServletResponse.SC_FORBIDDEN);
                    return;
                }
            } else {
                TourVisaCounselor tourVisaCounselor = tourVisaCounselorService.getById(id);
                if (tourVisaCounselor == null) {
                    ResponseResult.output(HttpServletResponse.SC_NOT_FOUND);
                    return;
                }
                String fieldJsonData = (String) ReflectUtil.getFieldValue(tourVisaCounselor, fieldName);
                if (fieldJsonData == null && !cacheHelper.existSessionUploadFile(filename)) {
                    ResponseResult.output(HttpServletResponse.SC_BAD_REQUEST);
                    return;
                }
                if (!BaseUpDownloader.containFile(fieldJsonData, filename)
                        && !cacheHelper.existSessionUploadFile(filename)) {
                    ResponseResult.output(HttpServletResponse.SC_FORBIDDEN);
                    return;
                }
            }
            UploadStoreInfo storeInfo = MyModelUtil.getUploadStoreInfo(TourVisaCounselor.class, fieldName);
            if (!storeInfo.isSupportUpload()) {
                ResponseResult.output(HttpServletResponse.SC_NOT_IMPLEMENTED,
                        ResponseResult.error(ErrorCodeEnum.INVALID_UPLOAD_FIELD));
                return;
            }
            BaseUpDownloader upDownloader = upDownloaderFactory.get(storeInfo.getStoreType());
            upDownloader.doDownload(appConfig.getUploadFileBaseDir(),
                    TourVisaCounselor.class.getSimpleName(), fieldName, filename, asImage, response);
        } catch (Exception e) {
            response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 文件上传操作。
     *
     * @param fieldName  上传文件名。
     * @param asImage    是否作为图片上传。如果是图片，今后下载的时候无需权限验证。否则就是附件上传，下载时需要权限验证。
     * @param uploadFile 上传文件对象。
     */
    @SaCheckPermission("tourVisaCounselor.view")
    @OperationLog(type = SysOperationLogType.UPLOAD, saveResponse = false)
    @PostMapping("/upload")
    public void upload(
            @RequestParam String fieldName,
            @RequestParam Boolean asImage,
            @RequestParam("uploadFile") MultipartFile uploadFile) throws IOException {
        UploadStoreInfo storeInfo = MyModelUtil.getUploadStoreInfo(TourVisaCounselor.class, fieldName);
        // 这里就会判断参数中指定的字段，是否支持上传操作。
        if (!storeInfo.isSupportUpload()) {
            ResponseResult.output(HttpServletResponse.SC_FORBIDDEN,
                    ResponseResult.error(ErrorCodeEnum.INVALID_UPLOAD_FIELD));
            return;
        }
        // 根据字段注解中的存储类型，通过工厂方法获取匹配的上传下载实现类，从而解耦。
        BaseUpDownloader upDownloader = upDownloaderFactory.get(storeInfo.getStoreType());
        UploadResponseInfo responseInfo = upDownloader.doUpload(null,
                appConfig.getUploadFileBaseDir(), TourVisaCounselor.class.getSimpleName(), fieldName, asImage, uploadFile);
        if (Boolean.TRUE.equals(responseInfo.getUploadFailed())) {
            ResponseResult.output(HttpServletResponse.SC_FORBIDDEN,
                    ResponseResult.error(ErrorCodeEnum.UPLOAD_FAILED, responseInfo.getErrorMessage()));
            return;
        }
        cacheHelper.putSessionUploadFile(responseInfo.getFilename());
        ResponseResult.output(ResponseResult.success(responseInfo));
    }

    /**
     * 以字典形式返回全部签证顾问信息数据集合。字典的键值为[id, counselorName]。
     * 白名单接口，登录用户均可访问。
     *
     * @param filter 过滤对象。
     * @return 应答结果对象，包含的数据为 List<Map<String, String>>，map中包含两条记录，key的值分别是id和name，value对应具体数据。
     */
    @GetMapping("/listDict")
    public ResponseResult<List<Map<String, Object>>> listDict(@ParameterObject TourVisaCounselorDto filter) {
        List<TourVisaCounselor> resultList =
                tourVisaCounselorService.getListByFilter(MyModelUtil.copyTo(filter, TourVisaCounselor.class));
        return ResponseResult.success(
                MyCommonUtil.toDictDataList(resultList, TourVisaCounselor::getId, TourVisaCounselor::getCounselorName));
    }

    /**
     * 根据字典Id集合，获取查询后的字典数据。
     *
     * @param dictIds 字典Id集合。
     * @return 应答结果对象，包含字典形式的数据集合。
     */
    @GetMapping("/listDictByIds")
    public ResponseResult<List<Map<String, Object>>> listDictByIds(@RequestParam List<Long> dictIds) {
        List<TourVisaCounselor> resultList = tourVisaCounselorService.getInList(new HashSet<>(dictIds));
        return ResponseResult.success(
                MyCommonUtil.toDictDataList(resultList, TourVisaCounselor::getId, TourVisaCounselor::getCounselorName));
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourVisaCounselor originalTourVisaCounselor = tourVisaCounselorService.getById(id);
        if (originalTourVisaCounselor == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourVisaCounselorService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
