package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.additional.config.ApplicationConfig;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.tourism.webadmin.back.dto.TourVisaTypeDto;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.tourism.webadmin.back.model.TourVisaType;
import com.tourism.webadmin.back.service.TourVisaTypeService;
import com.tourism.webadmin.back.vo.TourVisaTypeVo;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;

/**
 * 签证类型信息操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "签证类型信息管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourVisaType")
public class TourVisaTypeController {

    @Autowired
    private ApplicationConfig appConfig;
    @Autowired
    private TourVisaTypeService tourVisaTypeService;

    /**
     * 新增签证类型信息数据。
     *
     * @param tourVisaTypeDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"tourVisaTypeDto.id", "tourVisaTypeDto.searchString"})
    @SaCheckPermission("tourVisaType.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody TourVisaTypeDto tourVisaTypeDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourVisaTypeDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourVisaType tourVisaType = MyModelUtil.copyTo(tourVisaTypeDto, TourVisaType.class);
        tourVisaType = tourVisaTypeService.saveNew(tourVisaType);
        return ResponseResult.success(tourVisaType.getId());
    }

    /**
     * 更新签证类型信息数据。
     *
     * @param tourVisaTypeDto 更新对象。
     * @return 应答结果对象。
     */
    @ApiOperationSupport(ignoreParameters = {"tourVisaTypeDto.searchString"})
    @SaCheckPermission("tourVisaType.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourVisaTypeDto tourVisaTypeDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourVisaTypeDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourVisaType tourVisaType = MyModelUtil.copyTo(tourVisaTypeDto, TourVisaType.class);
        TourVisaType originalTourVisaType = tourVisaTypeService.getById(tourVisaType.getId());
        if (originalTourVisaType == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourVisaTypeService.update(tourVisaType, originalTourVisaType)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除签证类型信息数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourVisaType.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除签证类型信息数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourVisaType.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的签证类型信息列表。
     *
     * @param tourVisaTypeDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourVisaType.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourVisaTypeVo>> list(
            @MyRequestBody TourVisaTypeDto tourVisaTypeDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourVisaType tourVisaTypeFilter = MyModelUtil.copyTo(tourVisaTypeDtoFilter, TourVisaType.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourVisaType.class);
        List<TourVisaType> tourVisaTypeList =
                tourVisaTypeService.getTourVisaTypeListWithRelation(tourVisaTypeFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourVisaTypeList, TourVisaTypeVo.class));
    }

    /**
     * 导入主表数据列表。
     *
     * @param importFile 上传的文件，目前仅仅支持xlsx和xls两种格式。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourVisaType.import")
    @OperationLog(type = SysOperationLogType.IMPORT)
    @PostMapping("/import")
    public ResponseResult<Void> importBatch(
            @RequestParam Boolean skipHeader,
            @RequestParam("importFile") MultipartFile importFile) throws IOException {
        String filename = ImportUtil.saveImportFile(appConfig.getUploadFileBaseDir(), null, importFile);
        // 这里可以指定需要忽略导入的字段集合。如创建时间、创建人、更新时间、更新人、主键Id和逻辑删除，
        // 以及一些存在缺省值且无需导入的字段。其中主键字段和逻辑删除字段不需要在这里设置，批量插入逻辑会自动处理的。
        Set<String> ignoreFieldSet = new HashSet<>();
        ignoreFieldSet.add("createUserId");
        ignoreFieldSet.add("createTime");
        ignoreFieldSet.add("updateUserId");
        ignoreFieldSet.add("updateTime");
        List<ImportUtil.ImportHeaderInfo> headerInfoList = ImportUtil.makeHeaderInfoList(TourVisaType.class, ignoreFieldSet);
        // 下面是导入时需要注意的地方，如果我们缺省生成的代码，与实际情况存在差异，请手动修改。
        // 1. 头信息数据字段，我们只是根据当前的主表实体对象生成了缺省数组，开发者可根据实际情况，对headerInfoList进行修改。
        ImportUtil.ImportHeaderInfo[] headerInfos = headerInfoList.toArray(new ImportUtil.ImportHeaderInfo[]{});
        // 2. 这里需要根据实际情况决定，导入文件中第一行是否为中文头信息，如果是可以跳过。这里我们默认为true。
        // 这里根据自己的实际需求，为doImport的最后一个参数，传递需要进行字典转换的字段集合。
        // 注意，集合中包含需要翻译的Java字段名，如: gradeId。
        Set<String> translatedDictFieldSet = new HashSet<>();
        List<TourVisaType> dataList =
                ImportUtil.doImport(headerInfos, skipHeader, filename, TourVisaType.class, translatedDictFieldSet);
        tourVisaTypeService.saveNewBatch(dataList, -1);
        return ResponseResult.success();
    }

    /**
     * 导出符合过滤条件的签证类型信息列表。
     *
     * @param tourVisaTypeDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @throws IOException 文件读写失败。
     */
    @SaCheckPermission("tourVisaType.export")
    @OperationLog(type = SysOperationLogType.EXPORT, saveResponse = false)
    @PostMapping("/export")
    public void export(
            @MyRequestBody TourVisaTypeDto tourVisaTypeDtoFilter,
            @MyRequestBody MyOrderParam orderParam) throws IOException {
        TourVisaType tourVisaTypeFilter = MyModelUtil.copyTo(tourVisaTypeDtoFilter, TourVisaType.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourVisaType.class);
        List<TourVisaType> resultList =
                tourVisaTypeService.getTourVisaTypeListWithRelation(tourVisaTypeFilter, orderBy);
        // 导出文件的标题数组
        // NOTE: 下面的代码中仅仅导出了主表数据，主表聚合计算数据和主表关联字典的数据。
        // 一对一从表数据的导出，可根据需要自行添加。如：headerMap.put("slaveFieldName.xxxField", "标题名称")
        Map<String, String> headerMap = new LinkedHashMap<>(8);
        headerMap.put("id", "签证类型id");
        headerMap.put("title", "签证类型名称");
        headerMap.put("recommand", "签证类型介绍");
        headerMap.put("deleteFlag", "逻辑删除 1正常 -1已删除");
        headerMap.put("createUserId", "创建人id");
        headerMap.put("createTime", "创建时间");
        headerMap.put("updateUserId", "更新人id");
        headerMap.put("updateTime", "更新时间");
        ExportUtil.doExport(resultList, headerMap, "tourVisaType.xlsx");
    }

    /**
     * 查看指定签证类型信息对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourVisaType.view")
    @GetMapping("/view")
    public ResponseResult<TourVisaTypeVo> view(@RequestParam Long id) {
        TourVisaType tourVisaType = tourVisaTypeService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourVisaType == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourVisaTypeVo tourVisaTypeVo = MyModelUtil.copyTo(tourVisaType, TourVisaTypeVo.class);
        return ResponseResult.success(tourVisaTypeVo);
    }
    /**
     * 以字典形式返回全部签证类型信息数据集合。字典的键值为[id, title]。
     * 白名单接口，登录用户均可访问。
     *
     * @param filter 过滤对象。
     * @return 应答结果对象，包含的数据为 List<Map<String, String>>，map中包含两条记录，key的值分别是id和name，value对应具体数据。
     */
    @GetMapping("/listDict")
    public ResponseResult<List<Map<String, Object>>> listDict(@ParameterObject TourVisaTypeDto filter) {
        List<TourVisaType> resultList =
                tourVisaTypeService.getListByFilter(MyModelUtil.copyTo(filter, TourVisaType.class));
        return ResponseResult.success(
                MyCommonUtil.toDictDataList(resultList, TourVisaType::getId, TourVisaType::getTitle));
    }

    /**
     * 根据字典Id集合，获取查询后的字典数据。
     *
     * @param dictIds 字典Id集合。
     * @return 应答结果对象，包含字典形式的数据集合。
     */
    @GetMapping("/listDictByIds")
    public ResponseResult<List<Map<String, Object>>> listDictByIds(@RequestParam List<Long> dictIds) {
        List<TourVisaType> resultList = tourVisaTypeService.getInList(new HashSet<>(dictIds));
        return ResponseResult.success(
                MyCommonUtil.toDictDataList(resultList, TourVisaType::getId, TourVisaType::getTitle));
    }
    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourVisaType originalTourVisaType = tourVisaTypeService.getById(id);
        if (originalTourVisaType == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourVisaTypeService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
