package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 旅游项目日历价格操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "旅游项目日历价格管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourismDatePrice")
public class TourismDatePriceController {

    @Autowired
    private TourismDatePriceService tourismDatePriceService;

    /**
     * 新增旅游项目日历价格数据。
     *
     * @param tourismDatePriceDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {
            "tourismDatePriceDto.id",
            "tourismDatePriceDto.departureDateStart",
            "tourismDatePriceDto.departureDateEnd"})
    @SaCheckPermission("tourismDatePrice.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<String> add(@MyRequestBody TourismDatePriceDto tourismDatePriceDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourismDatePriceDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourismDatePrice tourismDatePrice = MyModelUtil.copyTo(tourismDatePriceDto, TourismDatePrice.class);
        // 验证关联Id的数据合法性
        CallResult callResult = tourismDatePriceService.verifyRelatedData(tourismDatePrice, null);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        tourismDatePrice = tourismDatePriceService.saveNew(tourismDatePrice);
        return ResponseResult.success(tourismDatePrice.getId());
    }

    /**
     * 更新旅游项目日历价格数据。
     *
     * @param tourismDatePriceDto 更新对象。
     * @return 应答结果对象。
     */
    @ApiOperationSupport(ignoreParameters = {
            "tourismDatePriceDto.departureDateStart",
            "tourismDatePriceDto.departureDateEnd"})
    @SaCheckPermission("tourismDatePrice.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourismDatePriceDto tourismDatePriceDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourismDatePriceDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourismDatePrice tourismDatePrice = MyModelUtil.copyTo(tourismDatePriceDto, TourismDatePrice.class);
        TourismDatePrice originalTourismDatePrice = tourismDatePriceService.getById(tourismDatePrice.getId());
        if (originalTourismDatePrice == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        // 验证关联Id的数据合法性
        CallResult callResult = tourismDatePriceService.verifyRelatedData(tourismDatePrice, originalTourismDatePrice);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        if (!tourismDatePriceService.update(tourismDatePrice, originalTourismDatePrice)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除旅游项目日历价格数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourismDatePrice.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除旅游项目日历价格数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourismDatePrice.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的旅游项目日历价格列表。
     *
     * @param tourismDatePriceDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourismDatePrice.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourismDatePriceVo>> list(
            @MyRequestBody TourismDatePriceDto tourismDatePriceDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourismDatePrice tourismDatePriceFilter = MyModelUtil.copyTo(tourismDatePriceDtoFilter, TourismDatePrice.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourismDatePrice.class);
        List<TourismDatePrice> tourismDatePriceList =
                tourismDatePriceService.getTourismDatePriceListWithRelation(tourismDatePriceFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourismDatePriceList, TourismDatePriceVo.class));
    }

    /**
     * 查看指定旅游项目日历价格对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourismDatePrice.view")
    @GetMapping("/view")
    public ResponseResult<TourismDatePriceVo> view(@RequestParam String id) {
        TourismDatePrice tourismDatePrice = tourismDatePriceService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourismDatePrice == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourismDatePriceVo tourismDatePriceVo = MyModelUtil.copyTo(tourismDatePrice, TourismDatePriceVo.class);
        return ResponseResult.success(tourismDatePriceVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourismDatePrice originalTourismDatePrice = tourismDatePriceService.getById(id);
        if (originalTourismDatePrice == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourismDatePriceService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
