package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 发布游记的文本框操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "发布游记的文本框管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourismTravelNotesContentWrite")
public class TourismTravelNotesContentWriteController {

    @Autowired
    private TourismTravelNotesContentWriteService tourismTravelNotesContentWriteService;

    /**
     * 新增发布游记的文本框数据。
     *
     * @param tourismTravelNotesContentWriteDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"tourismTravelNotesContentWriteDto.id"})
    @SaCheckPermission("tourismTravelNotesContentWrite.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody TourismTravelNotesContentWriteDto tourismTravelNotesContentWriteDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourismTravelNotesContentWriteDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourismTravelNotesContentWrite tourismTravelNotesContentWrite = MyModelUtil.copyTo(tourismTravelNotesContentWriteDto, TourismTravelNotesContentWrite.class);
        tourismTravelNotesContentWrite = tourismTravelNotesContentWriteService.saveNew(tourismTravelNotesContentWrite);
        return ResponseResult.success(tourismTravelNotesContentWrite.getId());
    }

    /**
     * 更新发布游记的文本框数据。
     *
     * @param tourismTravelNotesContentWriteDto 更新对象。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourismTravelNotesContentWrite.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourismTravelNotesContentWriteDto tourismTravelNotesContentWriteDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourismTravelNotesContentWriteDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourismTravelNotesContentWrite tourismTravelNotesContentWrite = MyModelUtil.copyTo(tourismTravelNotesContentWriteDto, TourismTravelNotesContentWrite.class);
        TourismTravelNotesContentWrite originalTourismTravelNotesContentWrite = tourismTravelNotesContentWriteService.getById(tourismTravelNotesContentWrite.getId());
        if (originalTourismTravelNotesContentWrite == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourismTravelNotesContentWriteService.update(tourismTravelNotesContentWrite, originalTourismTravelNotesContentWrite)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除发布游记的文本框数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourismTravelNotesContentWrite.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除发布游记的文本框数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourismTravelNotesContentWrite.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的发布游记的文本框列表。
     *
     * @param tourismTravelNotesContentWriteDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourismTravelNotesContentWrite.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourismTravelNotesContentWriteVo>> list(
            @MyRequestBody TourismTravelNotesContentWriteDto tourismTravelNotesContentWriteDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourismTravelNotesContentWrite tourismTravelNotesContentWriteFilter = MyModelUtil.copyTo(tourismTravelNotesContentWriteDtoFilter, TourismTravelNotesContentWrite.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourismTravelNotesContentWrite.class);
        List<TourismTravelNotesContentWrite> tourismTravelNotesContentWriteList =
                tourismTravelNotesContentWriteService.getTourismTravelNotesContentWriteListWithRelation(tourismTravelNotesContentWriteFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourismTravelNotesContentWriteList, TourismTravelNotesContentWriteVo.class));
    }

    /**
     * 查看指定发布游记的文本框对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourismTravelNotesContentWrite.view")
    @GetMapping("/view")
    public ResponseResult<TourismTravelNotesContentWriteVo> view(@RequestParam Long id) {
        TourismTravelNotesContentWrite tourismTravelNotesContentWrite = tourismTravelNotesContentWriteService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourismTravelNotesContentWrite == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourismTravelNotesContentWriteVo tourismTravelNotesContentWriteVo = MyModelUtil.copyTo(tourismTravelNotesContentWrite, TourismTravelNotesContentWriteVo.class);
        return ResponseResult.success(tourismTravelNotesContentWriteVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourismTravelNotesContentWrite originalTourismTravelNotesContentWrite = tourismTravelNotesContentWriteService.getById(id);
        if (originalTourismTravelNotesContentWrite == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourismTravelNotesContentWriteService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
