package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.additional.config.ApplicationConfig;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;

/**
 * 签证基础资料管理操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "签证基础资料管理管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/visaMetarialsList")
public class VisaMetarialsListController {

    @Autowired
    private ApplicationConfig appConfig;
    @Autowired
    private VisaMetarialsListService visaMetarialsListService;

    /**
     * 新增签证基础资料管理数据。
     *
     * @param visaMetarialsListDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"visaMetarialsListDto.id", "visaMetarialsListDto.searchString"})
    @SaCheckPermission("visaMetarialsList.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody VisaMetarialsListDto visaMetarialsListDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(visaMetarialsListDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        VisaMetarialsList visaMetarialsList = MyModelUtil.copyTo(visaMetarialsListDto, VisaMetarialsList.class);
        visaMetarialsList = visaMetarialsListService.saveNew(visaMetarialsList);
        return ResponseResult.success(visaMetarialsList.getId());
    }

    /**
     * 更新签证基础资料管理数据。
     *
     * @param visaMetarialsListDto 更新对象。
     * @return 应答结果对象。
     */
    @ApiOperationSupport(ignoreParameters = {"visaMetarialsListDto.searchString"})
    @SaCheckPermission("visaMetarialsList.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody VisaMetarialsListDto visaMetarialsListDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(visaMetarialsListDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        VisaMetarialsList visaMetarialsList = MyModelUtil.copyTo(visaMetarialsListDto, VisaMetarialsList.class);
        VisaMetarialsList originalVisaMetarialsList = visaMetarialsListService.getById(visaMetarialsList.getId());
        if (originalVisaMetarialsList == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!visaMetarialsListService.update(visaMetarialsList, originalVisaMetarialsList)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除签证基础资料管理数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("visaMetarialsList.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除签证基础资料管理数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("visaMetarialsList.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的签证基础资料管理列表。
     *
     * @param visaMetarialsListDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("visaMetarialsList.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<VisaMetarialsListVo>> list(
            @MyRequestBody VisaMetarialsListDto visaMetarialsListDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        VisaMetarialsList visaMetarialsListFilter = MyModelUtil.copyTo(visaMetarialsListDtoFilter, VisaMetarialsList.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, VisaMetarialsList.class);
        List<VisaMetarialsList> visaMetarialsListList =
                visaMetarialsListService.getVisaMetarialsListListWithRelation(visaMetarialsListFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(visaMetarialsListList, VisaMetarialsListVo.class));
    }

    /**
     * 导入主表数据列表。
     *
     * @param importFile 上传的文件，目前仅仅支持xlsx和xls两种格式。
     * @return 应答结果对象。
     */
    @SaCheckPermission("visaMetarialsList.import")
    @OperationLog(type = SysOperationLogType.IMPORT)
    @PostMapping("/import")
    public ResponseResult<Void> importBatch(
            @RequestParam Boolean skipHeader,
            @RequestParam("importFile") MultipartFile importFile) throws IOException {
        String filename = ImportUtil.saveImportFile(appConfig.getUploadFileBaseDir(), null, importFile);
        // 这里可以指定需要忽略导入的字段集合。如创建时间、创建人、更新时间、更新人、主键Id和逻辑删除，
        // 以及一些存在缺省值且无需导入的字段。其中主键字段和逻辑删除字段不需要在这里设置，批量插入逻辑会自动处理的。
        Set<String> ignoreFieldSet = new HashSet<>();
        ignoreFieldSet.add("createTime");
        ignoreFieldSet.add("createUserId");
        ignoreFieldSet.add("updateTime");
        ignoreFieldSet.add("updateUserId");
        List<ImportUtil.ImportHeaderInfo> headerInfoList = ImportUtil.makeHeaderInfoList(VisaMetarialsList.class, ignoreFieldSet);
        // 下面是导入时需要注意的地方，如果我们缺省生成的代码，与实际情况存在差异，请手动修改。
        // 1. 头信息数据字段，我们只是根据当前的主表实体对象生成了缺省数组，开发者可根据实际情况，对headerInfoList进行修改。
        ImportUtil.ImportHeaderInfo[] headerInfos = headerInfoList.toArray(new ImportUtil.ImportHeaderInfo[]{});
        // 2. 这里需要根据实际情况决定，导入文件中第一行是否为中文头信息，如果是可以跳过。这里我们默认为true。
        // 这里根据自己的实际需求，为doImport的最后一个参数，传递需要进行字典转换的字段集合。
        // 注意，集合中包含需要翻译的Java字段名，如: gradeId。
        Set<String> translatedDictFieldSet = new HashSet<>();
        List<VisaMetarialsList> dataList =
                ImportUtil.doImport(headerInfos, skipHeader, filename, VisaMetarialsList.class, translatedDictFieldSet);
        visaMetarialsListService.saveNewBatch(dataList, -1);
        return ResponseResult.success();
    }

    /**
     * 导出符合过滤条件的签证基础资料管理列表。
     *
     * @param visaMetarialsListDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @throws IOException 文件读写失败。
     */
    @SaCheckPermission("visaMetarialsList.export")
    @OperationLog(type = SysOperationLogType.EXPORT, saveResponse = false)
    @PostMapping("/export")
    public void export(
            @MyRequestBody VisaMetarialsListDto visaMetarialsListDtoFilter,
            @MyRequestBody MyOrderParam orderParam) throws IOException {
        VisaMetarialsList visaMetarialsListFilter = MyModelUtil.copyTo(visaMetarialsListDtoFilter, VisaMetarialsList.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, VisaMetarialsList.class);
        List<VisaMetarialsList> resultList =
                visaMetarialsListService.getVisaMetarialsListListWithRelation(visaMetarialsListFilter, orderBy);
        // 导出文件的标题数组
        // NOTE: 下面的代码中仅仅导出了主表数据，主表聚合计算数据和主表关联字典的数据。
        // 一对一从表数据的导出，可根据需要自行添加。如：headerMap.put("slaveFieldName.xxxField", "标题名称")
        Map<String, String> headerMap = new LinkedHashMap<>(9);
        headerMap.put("id", "基础材料id");
        headerMap.put("visaId", "签证商品id");
        headerMap.put("title", "基础材料名字");
        headerMap.put("content", "基础材料介绍");
        headerMap.put("deleteFlag", "是否逻辑删除");
        headerMap.put("createTime", "创建时间");
        headerMap.put("createUserId", "创建人id");
        headerMap.put("updateTime", "更新时间");
        headerMap.put("updateUserId", "更新人id");
        ExportUtil.doExport(resultList, headerMap, "visaMetarialsList.xlsx");
    }

    /**
     * 查看指定签证基础资料管理对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("visaMetarialsList.view")
    @GetMapping("/view")
    public ResponseResult<VisaMetarialsListVo> view(@RequestParam Long id) {
        VisaMetarialsList visaMetarialsList = visaMetarialsListService.getByIdWithRelation(id, MyRelationParam.full());
        if (visaMetarialsList == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        VisaMetarialsListVo visaMetarialsListVo = MyModelUtil.copyTo(visaMetarialsList, VisaMetarialsListVo.class);
        return ResponseResult.success(visaMetarialsListVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        VisaMetarialsList originalVisaMetarialsList = visaMetarialsListService.getById(id);
        if (originalVisaMetarialsList == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!visaMetarialsListService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
