package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.github.pagehelper.Page;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.dict.service.GlobalDictService;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.tourism.webadmin.back.dao.TourCarOrderMapper;
import com.tourism.webadmin.back.model.TourCarCategory;
import com.tourism.webadmin.back.model.TourCarDriver;
import com.tourism.webadmin.back.model.TourCarOrder;
import com.tourism.webadmin.back.service.TourCarContractMenuService;
import com.tourism.webadmin.back.service.TourCarOrderService;
import com.tourism.webadmin.upms.service.SysDeptService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 包车订单数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("tourCarOrderService")
public class TourCarOrderServiceImpl extends BaseService<TourCarOrder, Long> implements TourCarOrderService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourCarOrderMapper tourCarOrderMapper;
    @Autowired
    private TourCarContractMenuService tourCarContractMenuService;
    @Autowired
    private SysDeptService sysDeptService;
    @Autowired
    private GlobalDictService globalDictService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourCarOrder> mapper() {
        return tourCarOrderMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourCarOrder saveNew(TourCarOrder tourCarOrder) {
        tourCarOrderMapper.insert(this.buildDefaultValue(tourCarOrder));
        return tourCarOrder;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourCarOrder> tourCarOrderList) {
        if (CollUtil.isNotEmpty(tourCarOrderList)) {
            tourCarOrderList.forEach(this::buildDefaultValue);
            tourCarOrderMapper.insertList(tourCarOrderList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourCarOrder tourCarOrder, TourCarOrder originalTourCarOrder) {
        MyModelUtil.fillCommonsForUpdate(tourCarOrder, originalTourCarOrder);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourCarOrder> uw = this.createUpdateQueryForNullValue(tourCarOrder, tourCarOrder.getId());
        return tourCarOrderMapper.update(tourCarOrder, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourCarOrderMapper.deleteById(id) == 1;
    }

    @Override
    public List<TourCarOrder> getTourCarOrderList(TourCarOrder filter, String orderBy) {
        return tourCarOrderMapper.getTourCarOrderList(filter, orderBy);
    }

    @Override
    public List<TourCarOrder> getTourCarOrderListWithRelation(TourCarOrder filter, TourCarCategory tourCarCategoryFilter, TourCarDriver tourCarDriverFilter, String orderBy) {
        List<TourCarOrder> resultList =
                tourCarOrderMapper.getTourCarOrderListEx(filter, tourCarCategoryFilter, tourCarDriverFilter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyRelatedData(TourCarOrder tourCarOrder, TourCarOrder originalTourCarOrder) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(tourCarOrder, originalTourCarOrder, TourCarOrder::getStartPlaceId)
                && !tourCarContractMenuService.existId(tourCarOrder.getStartPlaceId())) {
            return CallResult.error(String.format(errorMessageFormat, "起点"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourCarOrder, originalTourCarOrder, TourCarOrder::getConnectSex)
                && !globalDictService.existDictItemFromCache("UserSex", tourCarOrder.getConnectSex())) {
            return CallResult.error(String.format(errorMessageFormat, "联系人性别"));
        }
        return CallResult.ok();
    }

    @Override
    public boolean verifyDriverOrder(TourCarOrder tourCarOrder) {
        LambdaQueryWrapper<TourCarOrder> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TourCarOrder::getDriverId, tourCarOrder.getDriverId());
        queryWrapper.and(
                i->i
                        .or(j->j.between(TourCarOrder::getStartTime, tourCarOrder.getStartTime(), tourCarOrder.getEndTime()))
                        .or(j->j.between(TourCarOrder::getEndTime, tourCarOrder.getStartTime(), tourCarOrder.getEndTime()))
                        .or(j->j.le(TourCarOrder::getStartTime, tourCarOrder.getStartTime())
                                .ge(TourCarOrder::getEndTime, tourCarOrder.getEndTime()))
        );
        List<TourCarOrder> tourCarOrders = tourCarOrderMapper.selectList(queryWrapper);
        if(CollUtil.isNotEmpty(tourCarOrders) && tourCarOrders.size() > 0)
            return true;
        return false;
    }

    private TourCarOrder buildDefaultValue(TourCarOrder tourCarOrder) {
        if (tourCarOrder.getId() == null) {
            tourCarOrder.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourCarOrder);
        tourCarOrder.setDataState(GlobalDeletedFlag.NORMAL);
        return tourCarOrder;
    }
}
