package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.model.constant.InterviewFlag;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 签证信息管理数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@MyDataSource(DataSourceType.MAIN)
@Service("tourVisaService")
public class TourVisaServiceImpl extends BaseService<TourVisa, Long> implements TourVisaService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourVisaMapper tourVisaMapper;
    @Autowired
    private TourVisaCounselorService tourVisaCounselorService;
    @Autowired
    private TourVisaTypeService tourVisaTypeService;
    @Autowired
    private TourCountryCodeService tourCountryCodeService;
    @Autowired
    private ChinaAreaService chinaAreaService;
    @Autowired
    private VisaMetarialsListService visaMetarialsListService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourVisa> mapper() {
        return tourVisaMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourVisa saveNew(TourVisa tourVisa) {
        tourVisaMapper.insert(this.buildDefaultValue(tourVisa));
        return tourVisa;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourVisa> tourVisaList) {
        if (CollUtil.isNotEmpty(tourVisaList)) {
            tourVisaList.forEach(this::buildDefaultValue);
            tourVisaMapper.insertList(tourVisaList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourVisa> tourVisaList, int batchSize) {
        if (CollUtil.isEmpty(tourVisaList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(tourVisaList.size(), start + batchSize);
            List<TourVisa> subList = tourVisaList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            tourVisaMapper.insertList(subList);
            if (end == tourVisaList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourVisa saveNewWithRelation(TourVisa tourVisa, JSONObject relationData) {
        this.saveNew(tourVisa);
        this.saveOrUpdateRelationData(tourVisa, relationData);
        return tourVisa;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourVisa tourVisa, TourVisa originalTourVisa) {
        MyModelUtil.fillCommonsForUpdate(tourVisa, originalTourVisa);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourVisa> uw = this.createUpdateQueryForNullValue(tourVisa, tourVisa.getId());
        return tourVisaMapper.update(tourVisa, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateWithRelation(
            TourVisa tourVisa, TourVisa originalTourVisa, JSONObject relationData) {
        // tourVisa 为空的时候，无需修改主表数据。
        if (tourVisa != null && !this.update(tourVisa, originalTourVisa)) {
            return false;
        }
        this.saveOrUpdateRelationData(originalTourVisa, relationData);
        return true;
    }

    private void saveOrUpdateRelationData(TourVisa tourVisa, JSONObject relationData) {
        List<VisaMetarialsList> visaMetarialsListList =
                relationData.getObject("visaMetarialsListList", new TypeReference<List<VisaMetarialsList>>() {});
        // 对于一对多更新，分为以下三步：
        // 1. 在关联从表中，删除掉与主表字段关联，但是又没有出现在本地更新中的数据。我们将这些数据视为需要删除的数据。
        // 2. 在本次更新数据列表中，如果从表的对象没有主键Id，我们视为新数据，可以批量插入。
        // 3. 在本次更新数据列表中，如果从表的对象存在主键Id，我们视为已有数据，逐条更新。
        if (visaMetarialsListList != null) {
            visaMetarialsListService.updateBatchByVisaId(tourVisa.getId(), visaMetarialsListList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        if (tourVisaMapper.deleteById(id) == 0) {
            return false;
        }
        visaMetarialsListService.removeByVisaId(id);
        return true;
    }

    @Override
    public List<TourVisa> getTourVisaList(TourVisa filter, String orderBy) {
        return tourVisaMapper.getTourVisaList(filter, orderBy);
    }

    @Override
    public List<TourVisa> getTourVisaListWithRelation(TourVisa filter, VisaMetarialsList visaMetarialsListFilter, String orderBy) {
        List<TourVisa> resultList =
                tourVisaMapper.getTourVisaListEx(filter, visaMetarialsListFilter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyImportList(List<TourVisa> dataList, Set<String> ignoreFieldSet) {
        CallResult callResult;
        if (!CollUtil.contains(ignoreFieldSet, "interviewFlag")) {
            callResult = verifyImportForConstDict(dataList, "interviewFlagDictMap", TourVisa::getInterviewFlag);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "hotValue")) {
            callResult = verifyImportForConstDict(dataList, "hotValueDictMap", TourVisa::getHotValue);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "id")) {
            callResult = verifyImportForDatasourceDict(dataList, "idDictMap", TourVisa::getId);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "visaTypeId")) {
            callResult = verifyImportForDatasourceDict(dataList, "visaTypeIdDictMap", TourVisa::getVisaTypeId);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "countryCodeId")) {
            callResult = verifyImportForDatasourceDict(dataList, "countryCodeIdDictMap", TourVisa::getCountryCodeId);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "issueVisaCode")) {
            callResult = verifyImportForDatasourceDict(dataList, "issueVisaCodeDictMap", TourVisa::getIssueVisaCode);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "counselorId")) {
            callResult = verifyImportForDatasourceDict(dataList, "counselorIdDictMap", TourVisa::getCounselorId);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        return CallResult.ok();
    }

    @Override
    public CallResult verifyRelatedData(TourVisa tourVisa, TourVisa originalTourVisa) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(tourVisa, originalTourVisa, TourVisa::getId)
                && !tourVisaCounselorService.existId(tourVisa.getId())) {
            return CallResult.error(String.format(errorMessageFormat, "签证id"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourVisa, originalTourVisa, TourVisa::getVisaTypeId)
                && !tourVisaTypeService.existId(tourVisa.getVisaTypeId())) {
            return CallResult.error(String.format(errorMessageFormat, "类型id"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourVisa, originalTourVisa, TourVisa::getCountryCodeId)
                && !tourCountryCodeService.existOne("countryCode", tourVisa.getCountryCodeId())) {
            return CallResult.error(String.format(errorMessageFormat, "国家编码"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourVisa, originalTourVisa, TourVisa::getIssueVisaCode)
                && !chinaAreaService.existId(tourVisa.getIssueVisaCode())) {
            return CallResult.error(String.format(errorMessageFormat, "出签地编码"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourVisa, originalTourVisa, TourVisa::getCounselorId)
                && !tourVisaCounselorService.existId(tourVisa.getCounselorId())) {
            return CallResult.error(String.format(errorMessageFormat, "负责顾问"));
        }
        return CallResult.ok();
    }

    private TourVisa buildDefaultValue(TourVisa tourVisa) {
        if (tourVisa.getId() == null) {
            tourVisa.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourVisa);
        tourVisa.setDeleteFlag(GlobalDeletedFlag.NORMAL);
        MyModelUtil.setDefaultValue(tourVisa, "img", "");
        MyModelUtil.setDefaultValue(tourVisa, "visaTitle", "");
        MyModelUtil.setDefaultValue(tourVisa, "countryCodeId", "");
        MyModelUtil.setDefaultValue(tourVisa, "embassyName", "");
        MyModelUtil.setDefaultValue(tourVisa, "embassyAddress", "");
        MyModelUtil.setDefaultValue(tourVisa, "validity", 0);
        MyModelUtil.setDefaultValue(tourVisa, "idleDays", 0);
        MyModelUtil.setDefaultValue(tourVisa, "processingDays", 0);
        MyModelUtil.setDefaultValue(tourVisa, "floorPrice", 0);
        MyModelUtil.setDefaultValue(tourVisa, "acceptanceRange", "");
        MyModelUtil.setDefaultValue(tourVisa, "entriesNum", 1);
        MyModelUtil.setDefaultValue(tourVisa, "interviewFlag", InterviewFlag.INTERVIEW_YES);
        return tourVisa;
    }
}
